#include "ExhibitorListViewer.h"
#include "StringMgr.h"
#include "DataMgr.h"
#include "MemoryMgr.h"
#include "Exhibitor.h"
#include "PluckerWriter.h"
#include "PDAVShowGuide.h"

static Boolean recordPartOfGroup(Char limitChar, Char firstChar)
{
    Boolean partOfGroup=false;
    if (limitChar == '*')
    {
        partOfGroup = true;
    }
    else if (limitChar == '0')
    {
        partOfGroup = ((firstChar >= '0') && (firstChar <= '9'));
    }
    else
    {
        char lc = limitChar | 0x20;
        firstChar |= 0x20;
        partOfGroup = (lc == firstChar);
    }
    return partOfGroup;
}

void ExhibitorListViewer_new(UInt16 startId, Char limitChar)
{
    MemHandle hrec;
    const UInt8 *prec;
    DmOpenRef db;
    UInt32 len;
    UInt16 paragraphLen = 0;
    LocalID indexRecordLocalID;
    LocalID homeHTMLRecordLocalID;
    LocalID urlHandlingRecordLocalID;
    LocalID urlDataRecordLocalID;
    const char *urlTransport = "exhibitor:";
    const char *urlTransport2 = "exhibitorlist:";
    const char *prevString = "Previous";
    const char *nextString = "Next";
    const char *headingString = "Browse by Alphabet:";
    UInt16 urlLen;
    UInt16 urlId = 0;
    UInt16 id;
    UInt16 itemsPerPage = 8;
    UInt16 endId = startId + itemsPerPage;

    Boolean firstOfGroup=false, lastOfGroup=true;

    int i;
    PluckerWriterReserved reserved[]=
    {
        {PLUCKER_RESERVED_HOME_HTML, 2},
        {PLUCKER_RESERVED_URL_HANDLING, 3},
    };
#define ARRAY_ELEMENTS(x) sizeof(x)/sizeof(x[0])

    db = DmOpenDatabaseByTypeCreator(SHOWGUIDE_EXHIBITORS_DB_TYPE,SHOWGUIDE_CREATOR,dmModeReadOnly);

    paragraphLen = (2*PLUCKER_CTL_LEN_SET_TEXT_COLOR)+(PLUCKER_CTL_LEN_SET_FONT+PLUCKER_CTL_LEN_SET_FONT+PLUCKER_CTL_LEN_NEW_LINE);
    paragraphLen += StrLen(headingString);


    /*
        Determine if this is the first page of the group. If it isn't, we
        need to put a Previous link on the page.
    */
    if (startId > 1)
    {
        char c;
        hrec = DmQueryRecord(db,startId-1);
        prec = MemHandleLock(hrec);
        c = prec[0];
        MemHandleUnlock(hrec);

        firstOfGroup = !recordPartOfGroup(limitChar, c);
    }
    else
    {
        firstOfGroup = true;
    }

    if (!firstOfGroup)
    {
        paragraphLen += StrLen(prevString);
        paragraphLen += PLUCKER_CTL_LEN_PAGE_LINK_BEGINS + PLUCKER_CTL_LEN_LINK_ENDS;
    }

    paragraphLen += PLUCKER_CTL_LEN_NEW_LINE;
    paragraphLen += PLUCKER_CTL_LEN_SET_MARGIN;

    for (id = startId; id <= endId; id++)
    {
        Boolean partOfGroup;
        hrec = DmQueryRecord(db,id);
        if (hrec)
        {
          prec = MemHandleLock(hrec);

          partOfGroup = recordPartOfGroup(limitChar, prec[0]);
          if (partOfGroup)
          {
              if (id == endId)
              {
                  lastOfGroup = false;
              }
              else
              {
                  int tmpLen = StrLen(prec);
                  if (tmpLen > 30)
                  {
                      paragraphLen += 33;
                  }
                  else
                  {
                      paragraphLen += tmpLen;
                  }

                  paragraphLen += PLUCKER_CTL_LEN_PAGE_LINK_BEGINS + PLUCKER_CTL_LEN_LINK_ENDS;
                  paragraphLen += PLUCKER_CTL_LEN_NEW_LINE;
                  urlId++;
              }
          }
          else
          {
              endId = id;
              break;
          }
          MemHandleUnlock(hrec);
        }
        else
        {
          endId = id;
          break;
        }
    }

    if (!lastOfGroup)
    {
        paragraphLen += StrLen(nextString);
        paragraphLen += PLUCKER_CTL_LEN_PAGE_LINK_BEGINS + PLUCKER_CTL_LEN_LINK_ENDS;
        paragraphLen += PLUCKER_CTL_LEN_SET_MARGIN;
    }

    /*
        Basic header and Record List
    */
    len = PLUCKER_WRITER_FILE_HEADER_LENGTH+2;
    len += PLUCKER_WRITER_RECORD_ENTRY_LENGTH * (2+(ARRAY_ELEMENTS(reserved)));

    indexRecordLocalID = len;
    len += PLUCKER_WRITER_INDEX_RECORD_LENGTH(ARRAY_ELEMENTS(reserved));

    homeHTMLRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += PLUCKER_WRITER_TEXT_DATA_RECORD_PARAGRAPH_HEADER_LENGTH*1;

    len += paragraphLen;

    /*
        Data size all taken in to account. Now work out the size for the URLs
    */
    urlHandlingRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += PLUCKER_WRITER_URL_HANDLING_DATA_RECORD_PARAGRAPH_HEADER_LENGTH;

    urlDataRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += 4; // 4 zero terminators for unused records

    urlLen = (StrLen(urlTransport)+5+1)*urlId;
    if (!firstOfGroup)
    {
        urlLen += StrLen(urlTransport2)+7+1;
    }

    if (!lastOfGroup)
    {
        urlLen += StrLen(urlTransport2)+7+1;
    }

    len += urlLen;

    urlId = 5;

    {

        UInt8 *dbPtr = MemPtrNew(len);
        UInt8 *tmp = dbPtr;

        tmp = PluckerWriter_fileHeader(tmp, 1, NULL);
        tmp = PluckerWriter_uInt16(tmp, 4); /* num record entries */
        tmp = PluckerWriter_recordEntry(tmp, indexRecordLocalID, 0, 0);
        tmp = PluckerWriter_recordEntry(tmp, homeHTMLRecordLocalID, 0, 1);
        tmp = PluckerWriter_recordEntry(tmp, urlHandlingRecordLocalID, 0, 2);
        tmp = PluckerWriter_recordEntry(tmp, urlDataRecordLocalID, 0, 3);
        tmp = PluckerWriter_indexRecord(tmp, ARRAY_ELEMENTS(reserved), reserved);
        tmp = PluckerWriter_dataRecordHeader(tmp, reserved[0].id, 1, paragraphLen+(PLUCKER_WRITER_TEXT_DATA_RECORD_PARAGRAPH_HEADER_LENGTH*1), PLUCKER_DATATYPE_PHTML);
        tmp = PluckerWriter_uInt16(tmp, paragraphLen);
        tmp = PluckerWriter_uInt16(tmp, 0);

        tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_BOLD);
        tmp = PluckerWriter_setTextColor(tmp, 0xff, 0x80, 0x00);
        tmp = PluckerWriter_writeZTS(tmp, headingString);
        tmp = PluckerWriter_setTextColor(tmp, 0x00, 0x00, 0x00);
        tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_REGULAR);
        tmp = PluckerWriter_newLine(tmp);

        if (!firstOfGroup)
        {
            tmp = PluckerWriter_pageLinkBegins(tmp, urlId++);
            tmp = PluckerWriter_writeZTS(tmp, prevString);
            tmp = PluckerWriter_linkEnds(tmp);
        }
        tmp = PluckerWriter_newLine(tmp);
        tmp = PluckerWriter_setMargin(tmp, 4, 0);

        for (id = startId; id < endId; id++)
        {
            int tmpLen;
            hrec = DmQueryRecord(db,id);
            prec = MemHandleLock(hrec);
            tmp = PluckerWriter_pageLinkBegins(tmp, urlId++);

            tmpLen = StrLen(prec);
            if (tmpLen > 30)
            {
                Char tmpStr[34];
                MemMove(tmpStr,prec,30);
                StrCopy(&tmpStr[30],"...");
                tmp = PluckerWriter_writeZTS(tmp, tmpStr);
            }
            else
            {
                tmp = PluckerWriter_writeZTS(tmp, prec);
            }

            tmp = PluckerWriter_linkEnds(tmp);
            tmp = PluckerWriter_newLine(tmp);
            MemHandleUnlock(hrec);
        }
        DmCloseDatabase(db);

        if (!lastOfGroup)
        {
            tmp = PluckerWriter_setMargin(tmp, 0, 0);
            tmp = PluckerWriter_pageLinkBegins(tmp, urlId++);
            tmp = PluckerWriter_writeZTS(tmp, nextString);
            tmp = PluckerWriter_linkEnds(tmp);            
        }

        /*
            URL handling record.
        */
        tmp = PluckerWriter_dataRecordHeader(tmp, reserved[1].id, 0, PLUCKER_WRITER_URL_HANDLING_DATA_RECORD_PARAGRAPH_HEADER_LENGTH, PLUCKER_DATATYPE_LINK_INDEX);
        tmp = PluckerWriter_uInt16(tmp, urlId);
        tmp = PluckerWriter_uInt16(tmp, 4);

        /*
            URL data record.
        */
        tmp = PluckerWriter_dataRecordHeader(tmp, 4, 0, urlLen+4, PLUCKER_DATATYPE_LINKS);
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 1)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 2)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 3)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 4)

        for (i = 5; i < urlId; i++)
        {
            Char buf[8];
            if ((!firstOfGroup) && (i == 5))
            {
                tmp = PluckerWriter_writeZTS(tmp, urlTransport2);
                StrPrintF(buf, "%05u,%c",startId - itemsPerPage,limitChar);
                tmp = PluckerWriter_writeZTS(tmp, buf);
                tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator
            }
            else if ((!lastOfGroup) && (i == urlId-1))
            {
                tmp = PluckerWriter_writeZTS(tmp, urlTransport2);
                StrPrintF(buf, "%05u,%c",startId + itemsPerPage,limitChar);
                tmp = PluckerWriter_writeZTS(tmp, buf);
                tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator
            }
            else
            {
                tmp = PluckerWriter_writeZTS(tmp, urlTransport);
                StrPrintF(buf, "%05u",startId + (i-(5+(firstOfGroup?0:1))));
                tmp = PluckerWriter_writeZTS(tmp, buf);
                tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator
            }
        }

        {
            int dbLen = tmp - dbPtr;
            dbLen = dbLen;
        }

        PluckerWriter_createDatabaseFromImage(dbPtr);
        MemPtrFree(dbPtr);
    }
}
