#include "ExhibitorViewer.h"
#include "DataMgr.h"
#include "StringMgr.h"
#include "MemoryMgr.h"
#include "PluckerWriter.h"
#include "Exhibitor.h"
#include "PDAVShowGuide.h"


static const char *static_headings[]=
{
    "Booth #: ",
    "Contact: ",
    "Phone: ",
    "Fax: ",
    "Email: ",
    "Website: ",
    "Description: ",
    "Company Focus: "
};

void ExhibitorViewer_new(UInt16 id)
{
    MemHandle hrec;
    const UInt8 *prec;
    DmOpenRef db;
    UInt32 len;
    UInt16 paragraphLen = 0;
    LocalID indexRecordLocalID;
    LocalID homeHTMLRecordLocalID;
    LocalID urlHandlingRecordLocalID;
    LocalID urlDataRecordLocalID;
    const char *urlTransport = "map:";
    UInt16 urlLen;

    const char *exhibitorData[8];
    Exhibitor exhibitor;
    int i;
    PluckerWriterReserved reserved[]=
    {
        {PLUCKER_RESERVED_HOME_HTML, 2},
        {PLUCKER_RESERVED_URL_HANDLING, 3},
    };
#define ARRAY_ELEMENTS(x) sizeof(x)/sizeof(x[0])

    db = DmOpenDatabaseByTypeCreator(SHOWGUIDE_EXHIBITORS_DB_TYPE,SHOWGUIDE_CREATOR,dmModeReadOnly);


    hrec = DmQueryRecord(db,id);
    prec = MemHandleLock(hrec);

    /*
        Basic header and Record List
    */
    len = PLUCKER_WRITER_FILE_HEADER_LENGTH+2;
    len += PLUCKER_WRITER_RECORD_ENTRY_LENGTH * (2+(ARRAY_ELEMENTS(reserved)));

    indexRecordLocalID = len;
    len += PLUCKER_WRITER_INDEX_RECORD_LENGTH(ARRAY_ELEMENTS(reserved));

    homeHTMLRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += PLUCKER_WRITER_TEXT_DATA_RECORD_PARAGRAPH_HEADER_LENGTH*1;

    paragraphLen = 2*PLUCKER_CTL_LEN_SET_TEXT_COLOR;
    paragraphLen += 9*(PLUCKER_CTL_LEN_SET_FONT+PLUCKER_CTL_LEN_SET_FONT+PLUCKER_CTL_LEN_NEW_LINE);

    for (i = 0; i < 8; i++)
    {
        paragraphLen += StrLen(static_headings[i]);
    }
    
    exhibitor.m_company = prec;

#define EXHIBITOR_FIELD(dest,src) exhibitor.dest = exhibitor.src + StrLen(exhibitor.src)+1; paragraphLen += (exhibitor.dest-exhibitor.src)-1;

    EXHIBITOR_FIELD(m_boothNumbers,m_company)
    paragraphLen += PLUCKER_CTL_LEN_PAGE_LINK_BEGINS + PLUCKER_CTL_LEN_LINK_ENDS;

    EXHIBITOR_FIELD(m_contact,m_boothNumbers)
    EXHIBITOR_FIELD(m_phone,m_contact)
    EXHIBITOR_FIELD(m_fax,m_phone)
    EXHIBITOR_FIELD(m_email,m_fax)
    EXHIBITOR_FIELD(m_website,m_email)
    EXHIBITOR_FIELD(m_wordDescription,m_website)
    EXHIBITOR_FIELD(m_companyFocus,m_wordDescription)

    exhibitor.m_coordinates = exhibitor.m_companyFocus + StrLen(exhibitor.m_companyFocus)+1;

    paragraphLen += StrLen(exhibitor.m_companyFocus);

    exhibitorData[0] = exhibitor.m_boothNumbers;
    exhibitorData[1] = exhibitor.m_contact;
    exhibitorData[2] = exhibitor.m_phone;
    exhibitorData[3] = exhibitor.m_fax;
    exhibitorData[4] = exhibitor.m_email;
    exhibitorData[5] = exhibitor.m_website;
    exhibitorData[6] = exhibitor.m_wordDescription;
    exhibitorData[7] = exhibitor.m_companyFocus;

    len += paragraphLen;

    /*
        Data size all taken in to account. Now work out the size for the URLs
    */

    urlHandlingRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += PLUCKER_WRITER_URL_HANDLING_DATA_RECORD_PARAGRAPH_HEADER_LENGTH;

    urlDataRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += 4; // 4 zero terminators for unused records
    urlLen = StrLen(urlTransport)+StrLen(exhibitor.m_coordinates)+1;
    len += urlLen;


    {


        UInt8 *dbPtr = MemPtrNew(len);
        UInt8 *tmp = dbPtr;
        tmp = PluckerWriter_fileHeader(tmp, 1, NULL);
        tmp = PluckerWriter_uInt16(tmp, 4); /* num record entries */
        tmp = PluckerWriter_recordEntry(tmp, indexRecordLocalID, 0, 0);
        tmp = PluckerWriter_recordEntry(tmp, homeHTMLRecordLocalID, 0, 1);
        tmp = PluckerWriter_recordEntry(tmp, urlHandlingRecordLocalID, 0, 2);
        tmp = PluckerWriter_recordEntry(tmp, urlDataRecordLocalID, 0, 3);
        tmp = PluckerWriter_indexRecord(tmp, ARRAY_ELEMENTS(reserved), reserved);
        tmp = PluckerWriter_dataRecordHeader(tmp, reserved[0].id, 1, paragraphLen+(PLUCKER_WRITER_TEXT_DATA_RECORD_PARAGRAPH_HEADER_LENGTH*1), PLUCKER_DATATYPE_PHTML);
        tmp = PluckerWriter_uInt16(tmp, paragraphLen);
        tmp = PluckerWriter_uInt16(tmp, 0);

        tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_BOLD);
        tmp = PluckerWriter_setTextColor(tmp, 0xff, 0x80, 0x00);
        tmp = PluckerWriter_writeZTS(tmp, exhibitor.m_company);
        tmp = PluckerWriter_setTextColor(tmp, 0x00, 0x00, 0x00);
        tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_REGULAR);
        tmp = PluckerWriter_newLine(tmp);

        for (i = 0; i < 8; i++)
        {
            tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_BOLD);
            tmp = PluckerWriter_writeZTS(tmp, static_headings[i]);
            tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_REGULAR);
            if (i == 0)
            {
                tmp = PluckerWriter_pageLinkBegins(tmp, 5);
            }
            tmp = PluckerWriter_writeZTS(tmp, exhibitorData[i]);
            if (i == 0)
            {
                tmp = PluckerWriter_linkEnds(tmp);
            }
            tmp = PluckerWriter_newLine(tmp);
        }

        /*
            URL handling record.
        */
        tmp = PluckerWriter_dataRecordHeader(tmp, reserved[1].id, 0, PLUCKER_WRITER_URL_HANDLING_DATA_RECORD_PARAGRAPH_HEADER_LENGTH, PLUCKER_DATATYPE_LINK_INDEX);
        tmp = PluckerWriter_uInt16(tmp, 5);
        tmp = PluckerWriter_uInt16(tmp, 4);

        /*
            URL data record.
        */
        tmp = PluckerWriter_dataRecordHeader(tmp, 4, 0, urlLen+4, PLUCKER_DATATYPE_LINKS);
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 1)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 2)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 3)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 4)
        tmp = PluckerWriter_writeZTS(tmp, urlTransport);
        tmp = PluckerWriter_writeZTS(tmp, exhibitor.m_coordinates);
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator

        PluckerWriter_createDatabaseFromImage(dbPtr);
        MemPtrFree(dbPtr);
    }

    MemHandleUnlock(hrec);
    DmCloseDatabase(db);
}