#include "MemoryMgr.h"
#include "PDAVShowGuide.h"
#include "Map.h"

static void Map_addBoothRecordNumber(Map *self, UInt32 i);
static void Map_addExhibitorRecordNumber(Map *self, UInt32 i);

Map *Map_new(void)
{
    int i;
    int numRecords;

    Map *self = (Map *)MemPtrNew(sizeof(Map));
    self->m_mapRect.left = self->m_mapRect.top = 99999;
    self->m_mapRect.right = self->m_mapRect.bottom = 0;

    self->m_boothRecordNumbersCount = 0;
    self->m_boothRecordNumbersSize = 4;
    self->m_boothRecordNumbers = (UInt32 *)MemPtrNew(sizeof(UInt32)*self->m_boothRecordNumbersSize);

    self->m_exhibitorRecordNumbersCount = 0;
    self->m_exhibitorRecordNumbersSize = 4;
    self->m_exhibitorRecordNumbers = (UInt32 *)MemPtrNew(sizeof(UInt32)*self->m_exhibitorRecordNumbersSize);

    self->m_database = DmOpenDatabaseByTypeCreator(SHOWGUIDE_MAP_DB_TYPE,SHOWGUIDE_CREATOR,dmModeReadOnly);

    /*
        Find all the records which hold map data
    */
    numRecords = DmNumRecords(self->m_database);

    for (i = 0; i < numRecords; i++)
    {
        MemHandle h = DmQueryRecord(self->m_database, i);
        int len = MemHandleSize(h);

        if (len > sizeof(MapHeader))
        {
            const UInt8 *p = (const UInt8 *) MemHandleLock(h);

            MapHeader mapHeader;

            mapHeader.m_unknown0 = *(p++)<<8;
            mapHeader.m_unknown0 |= *(p++);
            mapHeader.m_left = *(p++)<<8;
            mapHeader.m_left |= *(p++);
            mapHeader.m_top = *(p++)<<8;
            mapHeader.m_top |= *(p++);
            mapHeader.m_right = *(p++)<<8;
            mapHeader.m_right |= *(p++);
            mapHeader.m_bottom = *(p++)<<8;
            mapHeader.m_bottom |= *(p++);

            if ((*p == 0x52) || (*p == 0x53))
            {
                /*
                    This record has map data in it. Extend the map size if necessary
                */
                if (self->m_mapRect.left > mapHeader.m_left)
                {
                    self->m_mapRect.left = mapHeader.m_left;
                }

                if (self->m_mapRect.top > mapHeader.m_top)
                {
                    self->m_mapRect.top = mapHeader.m_top;
                }

                if (self->m_mapRect.right < mapHeader.m_right)
                {
                    self->m_mapRect.right = mapHeader.m_right;
                }

                if (self->m_mapRect.bottom < mapHeader.m_bottom)
                {
                    self->m_mapRect.bottom = mapHeader.m_bottom;
                }

                if (*p == 0x53)
                {
                    Map_addExhibitorRecordNumber(self, i);
                }
                else
                {
                    Map_addBoothRecordNumber(self, i);
                }
            }
            MemHandleUnlock(h);
        }
    }
    return self;
}

void Map_delete(Map *self)
{
    DmCloseDatabase(self->m_database);

    MemPtrFree(self->m_boothRecordNumbers);
    MemPtrFree(self->m_exhibitorRecordNumbers);
    MemPtrFree(self);
}

static void Map_addBoothRecordNumber(Map *self, UInt32 i)
{
    if (self->m_boothRecordNumbersCount == self->m_boothRecordNumbersSize)
    {
        UInt32 *tmp;
        if (self->m_boothRecordNumbersSize < 32)
        {
            self->m_boothRecordNumbersSize <<= 1;
        }
        else
        {
            self->m_boothRecordNumbersSize += 16;
        }
        tmp = (UInt32 *)MemPtrNew(sizeof(UInt32) * self->m_boothRecordNumbersSize);

        if (tmp == NULL)
        {
            /*
                FIXME: Error, die.
            */
        }
        MemMove(tmp, self->m_boothRecordNumbers, sizeof(UInt32)*self->m_boothRecordNumbersCount);
        MemPtrFree(self->m_boothRecordNumbers);
        self->m_boothRecordNumbers = tmp;
    }
    self->m_boothRecordNumbers[self->m_boothRecordNumbersCount++] = i;
}


static void Map_addExhibitorRecordNumber(Map *self, UInt32 i)
{
    if (self->m_exhibitorRecordNumbersCount == self->m_exhibitorRecordNumbersSize)
    {
        UInt32 *tmp;
        if (self->m_exhibitorRecordNumbersSize < 32)
        {
            self->m_exhibitorRecordNumbersSize <<= 1;
        }
        else
        {
            self->m_exhibitorRecordNumbersSize += 16;
        }
        tmp = (UInt32 *)MemPtrNew(sizeof(UInt32) * self->m_exhibitorRecordNumbersSize);

        if (tmp == NULL)
        {
            /*
                FIXME: Error, die.
            */
        }
        MemMove(tmp, self->m_exhibitorRecordNumbers, sizeof(UInt32)*self->m_exhibitorRecordNumbersCount);
        MemPtrFree(self->m_exhibitorRecordNumbers);
        self->m_exhibitorRecordNumbers = tmp;
    }
    self->m_exhibitorRecordNumbers[self->m_exhibitorRecordNumbersCount++] = i;
}


