#include "MapViewerForm.h"
#include "ExhibitorViewer.h"

#include "doclist.h"
#include "libraryform.h"
#include "history.h"
#include "util.h"
#include "control.h"
#include "hires.h"
#include "genericfile.h"

#include "resourceids.h"
#include "PDAVShowGuide.h"


Boolean MapViewerFormHandleEvent( EventType* event )
{
  UInt16  prevCoordSys;
  Boolean handled = false;

  switch (event->eType)
  {
    case frmLoadEvent:
    {
      FormType *ft = FrmGetFormPtr(event->data.frmLoad.formID);

      /*
        Need to do this here, as JumpToRecord wants to update the visited link status
        of the document after the call to FrmGotoForm.
      */
      CloseDocument();

      FrmSetActiveForm(ft);
      handled = true;
      break;
    }
    case frmOpenEvent:
    case frmUpdateEvent:
    {
      FormType *ft = FrmGetActiveForm();
      UInt16 gadgetIndex;
      RectangleType bounds;
      UInt32 i;
      Int16 updateLeftMap, updateRightMap, updateTopMap, updateBottomMap;
      MapViewer *self = theMapViewer;
      Map *theMap = self->m_map;
      FontID oldFont;
      Int16 charHeight;
      RectangleType clipRect;
      Coord gadgetLeft, gadgetTop;

      if ((event->eType != frmUpdateEvent) || (event->data.frmUpdate.updateCode != (frmRedrawUpdateCode+1)))
      {
        FrmDrawForm(ft);
      }

      GetControlBounds( ft, bmpZoomIn, ZOOMINCONTROL );
      GetControlBounds( ft, bmpZoomOut, ZOOMOUTCONTROL );

      WinGetClip(&clipRect);

      gadgetIndex = FrmGetObjectIndex(ft, frmMapViewerMapGadget);
      FrmGetObjectBounds(ft, gadgetIndex, &bounds);
      gadgetLeft = bounds.topLeft.x;
      gadgetTop = bounds.topLeft.y;

      WinSetClip(&bounds);

      if (event->eType == frmOpenEvent)
      {
        RECT r;
        r.left = bounds.topLeft.x;
        r.right = bounds.topLeft.x + bounds.extent.x;
        r.top = bounds.topLeft.y;
        r.bottom = bounds.topLeft.y + bounds.extent.y;
        MapViewer_initialise(self,&r);
      }
      else if (event->data.frmUpdate.updateCode == (frmRedrawUpdateCode+1))
      {
        RECT wr;
        RectangleType r;
        float fl,fr,ft,fb;

        fl = self->m_highlightRect.left*self->m_scale;
        fr = self->m_highlightRect.right*self->m_scale;
        ft = self->m_highlightRect.top*self->m_scale;
        fb = self->m_highlightRect.bottom*self->m_scale;

        wr.left = ((int)fl) + self->m_xOffset;
        wr.right = ((int)fr) + self->m_xOffset;
        wr.top = ((int)ft) + self->m_yOffset;
        wr.bottom = ((int)fb) + self->m_yOffset;

        if (0 > wr.left)
        {
          r.topLeft.x = bounds.topLeft.x;
        }
        else
        {
          r.topLeft.x = bounds.topLeft.x + wr.left;
        }

        if (0 > wr.top)
        {
          r.topLeft.y = bounds.topLeft.y;
        }
        else
        {
          r.topLeft.y = bounds.topLeft.y + wr.top;
        }

        if ((bounds.extent.x) > wr.right)
        {
          r.extent.x = wr.right-wr.left;
        }
        else
        {
          r.extent.x = bounds.topLeft.x + bounds.extent.x - r.topLeft.x;
        }

        if ((bounds.extent.y) > wr.bottom)
        {
          r.extent.y = wr.bottom-wr.top;
        }
        else
        {
          r.extent.y = bounds.topLeft.y + bounds.extent.y - r.topLeft.y;
        }

        WinSetClip(&r);
        WinEraseRectangle(&r, 0);
      }
      else
      {
        WinEraseRectangle(&bounds, 0);
      }

      oldFont = FntSetFont(stdFont);
      charHeight = FntCharHeight();

      updateLeftMap = (Int16)(((float) (-self->m_xOffset)) / self->m_scale);
      updateRightMap = (Int16)(((float) (bounds.extent.x - self->m_xOffset)) / self->m_scale);
      updateTopMap = (Int16)(((float) (- self->m_yOffset)) / self->m_scale);
      updateBottomMap = (Int16)(((float) (bounds.extent.y - self->m_yOffset)) / self->m_scale);

      for (i = 0; i < theMap->m_boothRecordNumbersCount; i++)
      {
          int recNum = theMap->m_boothRecordNumbers[i];
          MemHandle h = DmQueryRecord(theMap->m_database, recNum);
          UInt8 *rec=MemHandleLock(h);
          int len = MemHandleSize(h);
          const UInt8 *p = &rec[10];

          while (p < (&rec[len]))
          {
              MapToBoothNumber mapToBoothNumber;

              mapToBoothNumber.m_recordType = *(p++);
              mapToBoothNumber.m_recordLength = *(p++);
              mapToBoothNumber.m_left = *(p++)<<8;
              mapToBoothNumber.m_left |= *(p++);
              mapToBoothNumber.m_top = *(p++)<<8;
              mapToBoothNumber.m_top |= *(p++);
              mapToBoothNumber.m_right = *(p++)<<8;
              mapToBoothNumber.m_right |= *(p++);
              mapToBoothNumber.m_bottom = *(p++)<<8;
              mapToBoothNumber.m_bottom |= *(p++);
              mapToBoothNumber.m_unknown1 = *(p++)<<8;
              mapToBoothNumber.m_unknown1 |= *(p++);
              mapToBoothNumber.m_boothNumber = p; p += StrLen(p)+1;

              /*
                  Compare the rectangle to see if we're within the update rect.
              */
              if ((((Int16)mapToBoothNumber.m_left) <= updateRightMap) &&
                  (((Int16)mapToBoothNumber.m_right) >= updateLeftMap) &&
                  (((Int16)mapToBoothNumber.m_top) <= updateBottomMap) &&
                  (((Int16)mapToBoothNumber.m_bottom) >= updateTopMap))
              {
                  RECT r;
                  float fl,fr,ft,fb;

                  fl = mapToBoothNumber.m_left*self->m_scale;
                  fr = mapToBoothNumber.m_right*self->m_scale;
                  ft = mapToBoothNumber.m_top*self->m_scale;
                  fb = mapToBoothNumber.m_bottom*self->m_scale;

                  r.left = ((int)fl) + self->m_xOffset;
                  r.right = ((int)fr) + self->m_xOffset;
                  r.top = ((int)ft) + self->m_yOffset;
                  r.bottom = ((int)fb) + self->m_yOffset;

                  bounds.topLeft.x = gadgetLeft + r.left + 1;
                  bounds.topLeft.y = gadgetTop + r.top + 1;
                  bounds.extent.x = (r.right-r.left) - 1;
                  bounds.extent.y = (r.bottom-r.top) - 1;
                  
                  WinDrawRectangleFrame(simpleFrame, &bounds);

                  if ((bounds.extent.y - charHeight) >= 0);
                  {
                    int len = (p - mapToBoothNumber.m_boothNumber)-1;
                    Int16 w;
                    w = FntCharsWidth(mapToBoothNumber.m_boothNumber,len);
                    if ((bounds.extent.x - w) >= 0)
                    {
                      WinDrawChars(mapToBoothNumber.m_boothNumber,len,bounds.topLeft.x + ((bounds.extent.x - w)>>1), bounds.topLeft.y + ((bounds.extent.y - charHeight)>>1)); 
                    }
                  }
              }

              /*
                  Wait 'til we've drawn the text to skip any extra NUL padding, or it
                  will show up in the image.
              */
              if (((p-rec))&1) p++;
          }
          MemHandleUnlock(h);
      }

      if (theMapViewer->m_highlightOn)
      {
        RECT r;
        float fl,fr,ft,fb;

        fl = theMapViewer->m_highlightRect.left*self->m_scale;
        fr = theMapViewer->m_highlightRect.right*self->m_scale;
        ft = theMapViewer->m_highlightRect.top*self->m_scale;
        fb = theMapViewer->m_highlightRect.bottom*self->m_scale;

        r.left = ((int)fl) + self->m_xOffset;
        r.right = ((int)fr) + self->m_xOffset;
        r.top = ((int)ft) + self->m_yOffset;
        r.bottom = ((int)fb) + self->m_yOffset;

        bounds.topLeft.x = gadgetLeft + r.left;
        bounds.topLeft.y = gadgetTop + r.top;
        bounds.extent.x = r.right-r.left;
        bounds.extent.y = r.bottom-r.top;
        WinDrawRectangle(&bounds,0);

      }
      FntSetFont(oldFont);
      WinSetClip(&clipRect);
      handled = true;
      break;
    }
    case frmCloseEvent:
    {
      MapViewer_delete(theMapViewer);
      break;
    }
    case nilEvent:
    {
      MapViewer *self = theMapViewer;
      if (self->m_highlightEnabled)
      {
        UInt32 currTime;
        currTime = TimGetTicks();
        if (currTime > (self->m_highlightLastSwitch - self->m_highlightPeriod))
        {
          self->m_highlightOn = !self->m_highlightOn;
          FrmUpdateForm(frmMapViewer, frmRedrawUpdateCode+1);
          self->m_highlightLastSwitch = currTime;
        }
      }
      break;
    }
    case penMoveEvent:
    {
      FormType *ft = FrmGetActiveForm();
      UInt16 gadgetIndex;
      RectangleType bounds;
      gadgetIndex = FrmGetObjectIndex(ft, frmMapViewerMapGadget);
      FrmGetObjectBounds(ft, gadgetIndex, &bounds);

      if (RctPtInRectangle( event->screenX, event->screenY, &bounds ))
      {
        Int16 x, y;
        MapViewer *self = theMapViewer;

        x = event->screenX;
        y = event->screenY;

        if ((x != self->m_xMouseDown) || (y != self->m_yMouseDown))
        {
            RECT r;
            Int32 xx = (self->m_xMouseDown-x);
            Int32 yy = (self->m_yMouseDown-y);
            self->m_xCentre += (Int32)(xx/self->m_scale);
            self->m_yCentre += (Int32)(yy/self->m_scale);
            self->m_xMouseDown = x;
            self->m_yMouseDown = y;
            self->m_mouseMovedSinceButtonDown = true;

            r.left = bounds.topLeft.x;
            r.right = bounds.topLeft.x + bounds.extent.x;
            r.top = bounds.topLeft.y;
            r.bottom = bounds.topLeft.y + bounds.extent.y;

            MapViewer_centre(self, &r);
            FrmUpdateForm(frmMapViewer, frmRedrawUpdateCode);
        }
      }
      break;
    }
    case penUpEvent:
    {
      FormType *ft = FrmGetActiveForm();
      UInt16 gadgetIndex;
      RectangleType bounds;
      gadgetIndex = FrmGetObjectIndex(ft, frmMapViewerMapGadget);
      FrmGetObjectBounds(ft, gadgetIndex, &bounds);

      if (RctPtInRectangle( event->screenX, event->screenY, &bounds ))
      {
        if (!theMapViewer->m_mouseMovedSinceButtonDown)
        {
          Int16 x, y;
          Int16 screenX, screenY;
          float fx, fy;
          UInt32 i;
          Map *theMap = theMapViewer->m_map;
          Boolean broken = false;

          screenX = event->screenX-bounds.topLeft.x;
          screenY = event->screenY-bounds.topLeft.y;

          x = screenX;
          y = screenY;

          x -= theMapViewer->m_xOffset;
          y -= theMapViewer->m_yOffset;

          fx = x/theMapViewer->m_scale;
          fy = y/theMapViewer->m_scale;

          x = (Int32)fx;
          y = (Int32)fy;

          for (i = 0; (!broken) && (i < theMap->m_exhibitorRecordNumbersCount); i++)
          {
            int recNum = theMap->m_exhibitorRecordNumbers[i];
            MemHandle h = DmQueryRecord(theMap->m_database, recNum);
            UInt8 *rec=MemHandleLock(h);
            int len = MemHandleSize(h);
            const UInt8 *p = &rec[10];

            while (p < (&rec[len]))
            {
              MapToExhibitorName mapToExhibitorName;

              mapToExhibitorName.m_recordType = *(p++);
              mapToExhibitorName.m_recordLength = *(p++);
              mapToExhibitorName.m_left = *(p++)<<8;
              mapToExhibitorName.m_left |= *(p++);
              mapToExhibitorName.m_top = *(p++)<<8;
              mapToExhibitorName.m_top |= *(p++);
              mapToExhibitorName.m_right = *(p++)<<8;
              mapToExhibitorName.m_right |= *(p++);
              mapToExhibitorName.m_bottom = *(p++)<<8;
              mapToExhibitorName.m_bottom |= *(p++);
              mapToExhibitorName.m_exhibitorName = p; p += StrLen(p)+1;
              mapToExhibitorName.m_recordTypeAndExhibitorRecordNum = p; p += StrLen(p)+1;

              mapToExhibitorName.m_right -= 1;
              mapToExhibitorName.m_bottom -= 1;

              if ((x >= (Int16)mapToExhibitorName.m_left) &&
                  (x <= (Int16)mapToExhibitorName.m_right) &&
                  (y >= (Int16)mapToExhibitorName.m_top) &&
                  (y <= (Int16)mapToExhibitorName.m_bottom))
              {

                ListType *lt;
                FontID oldFont;
                Int16 w;
                Int16 px;

                oldFont=FntSetFont(stdFont);
                w = FntCharsWidth(mapToExhibitorName.m_exhibitorName,(mapToExhibitorName.m_recordTypeAndExhibitorRecordNum - mapToExhibitorName.m_exhibitorName)-1);
                w += 6;
                FntSetFont(oldFont);

                px = event->screenX - (w/2);
                if (px < 0)
                {
                  px = 0;
                }

                LstNewList ((void **)(&ft),frmMapViewerList,px,event->screenY,w,20,stdFont,1,frmMapViewerListTrigger);
                lt = FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmMapViewerList));
                LstSetListChoices(lt, (char **)(&mapToExhibitorName.m_exhibitorName), 1);
                LstSetSelection(lt, noListSelection);

                if (LstPopupList(lt) == 0)
                {

                  int type, id;
                  Char *circPos;
                  type = StrAToI(&mapToExhibitorName.m_recordTypeAndExhibitorRecordNum[1]);
                  circPos = StrChr(&mapToExhibitorName.m_recordTypeAndExhibitorRecordNum[1],'^');
                  if (circPos)
                  {
                    id = StrAToI(&circPos[1]);
                    ExhibitorViewer_new(id);

                    {
                      Int16 docIndex;
                      DocumentInfo *docInfo;

                      InitializeDocInfoList();
                      FindDocData( SHOWGUIDE_TEMP_NAME, ALL_ELEMENTS, &docIndex );
                      docInfo = DocInfo( docIndex );

                      if (docInfo != NULL)
                      {
                        OpenDocument( docInfo );
                        AddToHistoryExhibitor(id);

                        FrmGotoForm( GetMainFormId() );
                        broken = true;
                      }
                    }
                  }
                }

                FrmRemoveObject(&ft, FrmGetObjectIndex(ft, frmMapViewerList));
                break;
              }
              if (((p-rec))&1) p++;
            }
            MemHandleUnlock(h);
          }
        }
      }
      else
      {
        prevCoordSys = PalmSetCoordinateSystem( NATIVE );
        TranslateHiResPenEvents( event );
        HandlePenUp( event->screenX, event->screenY );
        PalmSetCoordinateSystem( prevCoordSys );
      }
      break;
    }
    case penDownEvent:
    {
      FormType *ft = FrmGetActiveForm();
      UInt16 gadgetIndex;
      RectangleType bounds;
      gadgetIndex = FrmGetObjectIndex(ft, frmMapViewerMapGadget);
      FrmGetObjectBounds(ft, gadgetIndex, &bounds);

      if (RctPtInRectangle( event->screenX, event->screenY, &bounds ))
      {
        MapViewer *self = theMapViewer;
        self->m_xMouseDown = event->screenX;
        self->m_yMouseDown = event->screenY;
        self->m_mouseMovedSinceButtonDown = false;
      }
      else
      {
        prevCoordSys = PalmSetCoordinateSystem( NATIVE );
        TranslateHiResPenEvents( event );
        HandlePenDown( event->screenX, event->screenY );
        PalmSetCoordinateSystem( prevCoordSys );
      }
      break;
    }
    default:
    {
      break;
    }
  }

  return handled;
}

MapViewer *theMapViewer=NULL;

MapViewer *MapViewer_new(void)
{
    MapViewer *self = (MapViewer *)MemPtrNew(sizeof(MapViewer));
    ErrFatalDisplayIf(!self,"Failed to allocate memory for MapViewer");
    MemSet(self, sizeof(MapViewer), 0);
    self->m_map = Map_new();
    self->m_highlightPeriod = SysTicksPerSecond()>>1;
    return self;
}

void MapViewer_delete(MapViewer *self)
{
  if (self)
  {
      Map_delete(self->m_map);
      MemPtrFree(self);
      if (self == theMapViewer)
      {
        theMapViewer = NULL;
      }
  }
}

void MapViewer_centre(MapViewer *self, const RECT *windowRect)
{
    Int32 windowWidth = windowRect->right - windowRect->left;
    Int32 windowHeight = windowRect->bottom - windowRect->top;

    self->m_xOffset = (windowWidth>>1) - (self->m_scale*self->m_xCentre);
    self->m_yOffset = (windowHeight>>1) - (self->m_scale*self->m_yCentre);
}

void MapViewer_initialise(MapViewer *self, const RECT *windowRect)
{
    Int32 windowWidth, windowHeight, mapWidth, mapHeight;
    float xScale, yScale;
    
    windowWidth = windowRect->right - windowRect->left;
    windowHeight = windowRect->bottom - windowRect->top;
    mapWidth = self->m_map->m_mapRect.right - self->m_map->m_mapRect.left;
    mapHeight = self->m_map->m_mapRect.bottom - self->m_map->m_mapRect.top;

    xScale = (float)windowWidth / (float)mapWidth;
    yScale = (float)windowHeight / (float)mapHeight;

    if (xScale > yScale)
    {
        self->m_scale = yScale;
    }
    else
    {
        self->m_scale = xScale;
    }

    self->m_scale *= 9;
    self->m_scale /= 10;

    self->m_xCentre = self->m_map->m_mapRect.left + (mapWidth>>1);
    self->m_yCentre = self->m_map->m_mapRect.top + (mapHeight>>1);

    MapViewer_centre(self, windowRect);
}

void MapViewer_setHighlight(MapViewer *self, RECT *highlightRect)
{
/*
  if (self->m_highlightTimer)
  {
    KillTimer(self->m_hwnd, self->m_highlightTimer);
  }
*/
  self->m_highlightRect = *highlightRect;
  self->m_highlightOn = true;
  self->m_highlightEnabled = true;
  self->m_highlightLastSwitch = TimGetTicks();
}

void MapViewer_mapRectToScreenRect(MapViewer *self, RECT *mapRect, RECT *screenRect)
{
    float fl,fr,ft,fb;

    fl = mapRect->left*self->m_scale;
    fr = mapRect->right*self->m_scale;
    ft = mapRect->top*self->m_scale;
    fb = mapRect->bottom*self->m_scale;

    screenRect->left = ((int)fl) + self->m_xOffset;
    screenRect->right = ((int)fr) + self->m_xOffset;
    screenRect->top = ((int)ft) + self->m_yOffset;
    screenRect->bottom = ((int)fb) + self->m_yOffset;
}

#if 0

            else if (lo == ID_MAP_ZOOMIN)
            {
                RECT r;
                GetWindowRect(hWnd, &r);
                theMapViewer->m_scale *= 1.5;
                MapViewer_centre(theMapViewer, &r);
                InvalidateRect(hWnd, NULL, TRUE);
            }
            else if (lo == ID_MAP_ZOOMOUT)
            {
                RECT r;
                GetWindowRect(hWnd, &r);
                theMapViewer->m_scale /= 1.5;
                MapViewer_centre(theMapViewer, &r);
                InvalidateRect(hWnd, NULL, TRUE);
            }
            break;
        }
        case WM_TIMER:
        {
          RECT r;
          MapViewer_mapRectToScreenRect(theMapViewer, &theMapViewer->m_highlightRect, &r);
          theMapViewer->m_highlightOn = !theMapViewer->m_highlightOn;
          InvalidateRect(theMapViewer->m_hwnd, &r, TRUE);
          break;
        }
        case WM_LBUTTONUP:
        {
            if (!theMapViewer->m_mouseMovedSinceButtonDown)
            {
                Int32 x, y;
                float fx, fy;
                UInt32 i;
                Map *theMap = theMapViewer->m_map;
                Boolean broken = false;

                x = (Int32) LOWORD(lParam);
                y = (Int32) HIWORD(lParam);

                x -= theMapViewer->m_xOffset;
                y -= theMapViewer->m_yOffset;

                fx = x/theMapViewer->m_scale;
                fy = y/theMapViewer->m_scale;

                x = (Int32)fx;
                y = (Int32)fy;

                for (i = 0; (!broken) && (i < theMap->m_exhibitorRecordNumbersCount); i++)
                {
                    int recNum = theMap->m_exhibitorRecordNumbers[i];
                    MemHandle h = DmQueryRecord(theMap->m_database, recNum);
                    UInt8 *rec=MemHandleLock(h);
                    int len = MemHandleSize(h);
                    const UInt8 *p = &rec[10];

                    while (p < (&rec[len]))
                    {
                        MapToExhibitorName mapToExhibitorName;

                        mapToExhibitorName.m_recordType = *(p++);
                        mapToExhibitorName.m_recordLength = *(p++);
                        mapToExhibitorName.m_left = *(p++)<<8;
                        mapToExhibitorName.m_left |= *(p++);
                        mapToExhibitorName.m_top = *(p++)<<8;
                        mapToExhibitorName.m_top |= *(p++);
                        mapToExhibitorName.m_right = *(p++)<<8;
                        mapToExhibitorName.m_right |= *(p++);
                        mapToExhibitorName.m_bottom = *(p++)<<8;
                        mapToExhibitorName.m_bottom |= *(p++);
                        mapToExhibitorName.m_exhibitorName = p; p += strlen(p)+1;
                        mapToExhibitorName.m_recordTypeAndExhibitorRecordNum = p; p += strlen(p)+1;


                        if ((x >= mapToExhibitorName.m_left) &&
                            (x <= mapToExhibitorName.m_right) &&
                            (y >= mapToExhibitorName.m_top) &&
                            (y <= mapToExhibitorName.m_bottom))
                        {
                            TCHAR buf[100];
                            RECT r;
                            HMENU menu = CreatePopupMenu();


                            {
                                int len = mapToExhibitorName.m_recordTypeAndExhibitorRecordNum - mapToExhibitorName.m_exhibitorName;
                                int i;
                                if (len > 99)
                                {
                                    len = 99;
                                }

                                for (i = 0; i < len; i++)
                                {
                                    buf[i] = mapToExhibitorName.m_exhibitorName[i];
                                }
                                buf[len] = '\0';

                            }


                            AppendMenu(menu,MF_STRING,1,buf);
                            GetWindowRect(hWnd, &r);
                            if (TrackPopupMenu(menu,
                                TPM_CENTERALIGN|TPM_VCENTERALIGN|TPM_RETURNCMD,
                                LOWORD(lParam)+r.left,
                                HIWORD(lParam)+r.top,
                                0,
                                hWnd,
                                NULL))
                            {
                                int type, id;
                                if (sscanf(mapToExhibitorName.m_recordTypeAndExhibitorRecordNum,"^%u^%u",&type, &id)==2)
                                {
                                  Document *document;
                                  ExhibitorViewer_new(id);

                                  document = Library_findDocument( theMapViewer->m_library, TEMPORARY_PLUCKER_DB_NAME );
                                  if( document != NULL )
                                  {
                                    OpenDocument( theMapViewer->m_library->hWnd, document );
                                    HistoryList_setHistoryItemExhibitor(HistoryList_add( theMapViewer->m_library->history), TEMPORARY_PLUCKER_DB_NAME, id);
                                    MapViewer_delete(theMapViewer);

                                    broken = true;
                                  }
                                }
                            }
                            break;
                        }
                        if (((p-rec))&1) p++;
                    }
                    MemHandleUnlock(h);
                }
            }
            break;
        }
        case WM_MOUSEMOVE:
        {
            break;
        }
    }
    return 0;
}


#endif
