#ifndef PLUCKERWRITER_H__505472E7_445D_1ED7_7D84_1EE03A3D1B9A
#define PLUCKERWRITER_H__505472E7_445D_1ED7_7D84_1EE03A3D1B9A

#include "PalmTypes.h"

#include "viewer.h"

typedef enum
{
    PLUCKER_CTL_PAGE_LINK_BEGINS = 0x0A,
    PLUCKER_CTL_PARAGRAPH_LINK_BEGINS = 0x0C,
    PLUCKER_CTL_LINK_ENDS = 0x08,
    PLUCKER_CTL_SET_FONT = 0x11,
    PLUCKER_CTL_EMBEDDED_IMAGE = 0x1A,
    PLUCKER_CTL_SET_MARGIN = 0x22,
    PLUCKER_CTL_ALIGNMENT_OF_TEXT = 0x29,
    PLUCKER_CTL_HORIZONTAL_RULE = 0x33,
    PLUCKER_CTL_NEW_LINE = 0x38,
    PLUCKER_CTL_ITALIC_TEXT_BEGINS = 0x40,
    PLUCKER_CTL_ITALIC_TEXT_ENDS = 0x48,
    PLUCKER_CTL_SET_TEXT_COLOR = 0x53,
    PLUCKER_CTL_MULTIPLE_EMBEDDED_IMAGE = 0x5C,
    PLUCKER_CTL_UNDERLINE_TEXT_BEGINS = 0x60,
    PLUCKER_CTL_UNDERLINE_TEXT_ENDS = 0x68,
    PLUCKER_CTL_STRIKE_THROUGH_TEXT_BEGINS = 0x70,
    PLUCKER_CTL_STRIKE_THROUGH_TEXT_ENDS = 0x78,
    PLUCKER_CTL_16_BIT_UNICODE_CHARACTER = 0x83,
    PLUCKER_CTL_32_BIT_UNICODE_CHARACTER = 0x85,

} PluckerWriterCtl;


typedef enum
{
    PLUCKER_CTL_LEN_PAGE_LINK_BEGINS = (2+2),
    PLUCKER_CTL_LEN_PARAGRAPH_LINK_BEGINS = (2+4),
    PLUCKER_CTL_LEN_LINK_ENDS = (2+0),
    PLUCKER_CTL_LEN_SET_FONT = (2+1),
    PLUCKER_CTL_LEN_EMBEDDED_IMAGE = (2+2),
    PLUCKER_CTL_LEN_SET_MARGIN = (2+2),
    PLUCKER_CTL_LEN_ALIGNMENT_OF_TEXT = (2+1),
    PLUCKER_CTL_LEN_HORIZONTAL_RULE = (2+3),
    PLUCKER_CTL_LEN_NEW_LINE = (2+0),
    PLUCKER_CTL_LEN_ITALIC_TEXT_BEGINS = (2+0),
    PLUCKER_CTL_LEN_ITALIC_TEXT_ENDS = (2+0),
    PLUCKER_CTL_LEN_SET_TEXT_COLOR = (2+3),
    PLUCKER_CTL_LEN_MULTIPLE_EMBEDDED_IMAGE = (2+4),
    PLUCKER_CTL_LEN_UNDERLINE_TEXT_BEGINS = (2+0),
    PLUCKER_CTL_LEN_UNDERLINE_TEXT_ENDS = (2+0),
    PLUCKER_CTL_LEN_STRIKE_THROUGH_TEXT_BEGINS = (2+0),
    PLUCKER_CTL_LEN_STRIKE_THROUGH_TEXT_ENDS = (2+0),
    PLUCKER_CTL_LEN_16_BIT_UNICODE_CHARACTER = (2+3),
    PLUCKER_CTL_LEN_32_BIT_UNICODE_CHARACTER = (2+5),

} PluckerWriterCtlLen;

typedef enum
{
    PLUCKER_FONT_REGULAR,
    PLUCKER_FONT_H1,
    PLUCKER_FONT_H2,
    PLUCKER_FONT_H3,
    PLUCKER_FONT_H4,
    PLUCKER_FONT_H5,
    PLUCKER_FONT_H6,
    PLUCKER_FONT_BOLD,
    PLUCKER_FONT_TT,
    PLUCKER_FONT_SMALL,

} PluckerWriterFontSpecifier;

typedef enum
{
    PLUCKER_ALIGNMENT_LEFT,
    PLUCKER_ALIGNMENT_RIGHT,
    PLUCKER_ALIGNMENT_CENTER,
    PLUCKER_ALIGNMENT_JUSTIFY,

} PluckerWriterAlignment;

typedef enum
{
    PLUCKER_RESERVED_HOME_HTML,
    PLUCKER_RESERVED_EXTERNAL_BOOKMARKS,
    PLUCKER_RESERVED_URL_HANDLING,
    PLUCKER_RESERVED_DEFAULT_CATEGORY,
    PLUCKER_RESERVED_ADDITIONAL_METADATA,

} PluckerWriterReservedName;

typedef struct
{
    PluckerWriterReservedName name;
    UInt16 id;

} PluckerWriterReserved;

typedef enum
{
    PLUCKER_DATATYPE_PHTML,
    PLUCKER_DATATYPE_PHTML_COMPRESSED,
    PLUCKER_DATATYPE_TBMP,
    PLUCKER_DATATYPE_TBMP_COMPRESSED,
    PLUCKER_DATATYPE_MAILTO,
    PLUCKER_DATATYPE_LINK_INDEX,
    PLUCKER_DATATYPE_LINKS,
    PLUCKER_DATATYPE_LINKS_COMPRESSED,
    PLUCKER_DATATYPE_BOOKMARKS,
    PLUCKER_DATATYPE_CATEGORY,
    PLUCKER_DATATYPE_METADATA,

} PluckerWriterDataRecordType;

#define PLUCKER_WRITER_FILE_HEADER_LENGTH (76)
#define PLUCKER_WRITER_RECORD_ENTRY_LENGTH (8)
#define PLUCKER_WRITER_INDEX_RECORD_LENGTH(x) (6+(4*x))
#define PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH (8)
#define PLUCKER_WRITER_TEXT_DATA_RECORD_PARAGRAPH_HEADER_LENGTH (4)
#define PLUCKER_WRITER_URL_HANDLING_DATA_RECORD_PARAGRAPH_HEADER_LENGTH (4)



extern MemPtr PluckerWriter_fileHeader(MemPtr p, UInt16 version, const Char *name) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_uInt8(MemPtr p, UInt8 value) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_uInt16(MemPtr p, UInt16 value) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_uInt32(MemPtr p, UInt32 value) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_recordEntry(MemPtr p, LocalID localID, UInt8 attributes, UInt32 uniqueID) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_indexRecord(MemPtr p, UInt16 numReserved, const PluckerWriterReserved *reserved) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_dataRecordHeader(MemPtr p, UInt16 uid, UInt16 paragraphs, UInt16 size, PluckerWriterDataRecordType type) SHOWGUIDE_SECTION;

extern MemPtr PluckerWriter_writeZTS(MemPtr p, const char *string) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_writeBlock(MemPtr p, MemPtr src, UInt16 len) SHOWGUIDE_SECTION;

extern MemPtr PluckerWriter_pageLinkBegins(MemPtr p, UInt16 recordID) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_paragraphLinkBegins(MemPtr p, UInt16 recordID, UInt16 paragraphOffset) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_linkEnds(MemPtr p) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_setFont(MemPtr p, PluckerWriterFontSpecifier fontSpecifier) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_embeddedImage(MemPtr p, UInt16 recordID) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_setMargin(MemPtr p, UInt8 leftMargin, UInt8 rightMargin) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_alignmentOfText(MemPtr p, PluckerWriterAlignment alignment) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_horizontalRule(MemPtr p, UInt8 height, UInt8 widthPixels, UInt8 widthPercent) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_newLine(MemPtr p) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_italicTextBegins(MemPtr p) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_italicTextEnds(MemPtr p) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_setTextColor(MemPtr p, UInt8 red, UInt8 green, UInt8 blue) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_multipleEmbeddedImage(MemPtr p, UInt16 alternateImageRecordID, UInt16 imageRecordID) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_underlineTextBegins(MemPtr p) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_underlineTextEnds(MemPtr p) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_strikeThroughTextBegins(MemPtr p) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_strikeThroughTextEnds(MemPtr p) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_16bitUnicodeCharacter(MemPtr p, UInt8 alternateTextLength, UInt16 unicodeCharacter) SHOWGUIDE_SECTION;
extern MemPtr PluckerWriter_32bitUnicodeCharacter(MemPtr p, UInt8 alternateTextLength, UInt32 unicodeCharacter) SHOWGUIDE_SECTION;

extern void PluckerWriter_createDatabaseFromImage(const UInt8 *image) SHOWGUIDE_SECTION;

#endif

