
#include "SessionSearchForm.h"
#include "SessionSearch.h"
#include "doclist.h"
#include "libraryform.h"
#include "history.h"
#include "util.h"
#include "control.h"
#include "hires.h"
#include "genericfile.h"

#include "resourceids.h"
#include "PDAVShowGuide.h"

/*
  FIXME: these should all be determined from databases.
*/
static const DateMapper dates[]=
{
  {"Any", 0, 0, 0},
  {"Saturday, May 27, 2006", 2006, 5, 27},
  {"Sunday, May 28, 2006", 2006, 5, 28},
  {"Monday, May 29, 2006", 2006, 5, 29},
  {"Tuesday, May 30, 2006", 2006, 5, 30},
  {"Wednesday, May 31, 2006", 2006, 5, 31},
  {"Thursday, June 1, 2006", 2006, 6, 1}
};

static const Track tracks[]=
{
  {"", "Any"},
  {"Knowledge","Knowledge Partner"},
  {"Specialty","Specialty"},
  {"HP2010","HP2010"},
  {"Primary","Primary Care"},
  {"Athletic","Athletic Trainer Credit"},
  {"CSAC","CSAC Top 10"},
};

static const Keyword keywords[]=
{
  {"", "Any"},
  {"ALRG", "Allergy/immunology"},
  {"ALTE", "Alternative/complementary medicine"},
  {"ANES", "Anesthesia"},
  {"CARD", "Cardiology"},
  {"CCME", "Critical care medicine"},
  {"COMM", "Communications"},
  {"DERM", "Dermatology"},
  {"DIVR", "Diversity"},
  {"EDUC", "Education"},
  {"ENTH", "Ears, nose, throat"},
  {"EMER", "Emergency medicine"},
  {"ENDO", "Endocrinology"},
  {"ETHI", "Ethics"},
  {"GAST", "Gastroenterology"},
  {"GENE", "Genetics"},
  {"GERI", "Geriatrics"},
  {"HEMO", "Hematology"},
  {"HIVA", "HIV/AIDS"},
  {"HPOL", "Health policy"},
  {"INFD", "Infectious disease"},
  {"LABM", "Lab medicine"},
  {"LEGI", "Legislative issues"},
  {"MALP", "Malpractice/Quality Assurance/Risk Management"},
  {"MISC", "Miscellaneous"},
  {"NEPH", "Nephrology"},
  {"NEUR", "Neurology"},
  {"NUTR", "Nutrition"},
  {"OBGY", "Obstetrics/gynecology"},
  {"OCCU", "Occupational and environmental medicine"},
  {"ONCO", "Oncology"},
  {"OPHT", "Ophthalmology"},
  {"ORTH", "Orthopedics"},
  {"PALL", "Palliative medicine"},
  {"PEDS", "Pediatrics"},
  {"PHAR", "Pharmacotherapeutics"},
  {"PREV", "Preventive medicine"},
  {"PSYC", "Psychiatry"},
  {"PUBL", "Public health"},
  {"PULM", "Pulmonology"},
  {"RADI", "Radiology"},
  {"REHB", "Physical medicine/rehabilitation"},
  {"REIM", "Reimbursement issues"},
  {"RHEU", "Rheumatology"},
  {"ROLE", "Role of a PA"},
  {"SUBA", "Substance abuse"},
  {"SURG", "Surgery"},
  {"TECH", "Technology"},
  {"TRAU", "Trauma"},
  {"TRAV", "Travel medicine"},
  {"UROL", "Urology"},
  {"WS", "Workshop"},
};

static Char **trackPtrs=NULL;
static Char **keywordPtrs=NULL;
static Char **datePtrs=NULL;

const int numTracks = sizeof(tracks)/sizeof(tracks[0]);
const int numKeywords = sizeof(keywords)/sizeof(keywords[0]);
const int numDates = sizeof(dates)/sizeof(dates[0]);

static SessionSearchForm sessionSearchData;

void SessionSearchForm_new(const UInt16 year, const UInt16 month, const UInt16 day, const Char *keyword, const Char *track, const Char *cme)
{
  sessionSearchData.m_year = year;
  sessionSearchData.m_month = month;
  sessionSearchData.m_day = day;
  StrCopy(sessionSearchData.m_keyword, keyword);
  StrCopy(sessionSearchData.m_track, track);
  StrCopy(sessionSearchData.m_cme, cme);

  FrmGotoForm(frmSessionSearch);
}

Boolean SessionSearchFormHandleEvent( EventType* event )
{
  UInt16  prevCoordSys;
  Boolean handled = false;

  switch (event->eType)
  {
    case frmLoadEvent:
    {
      FormType *ft = FrmGetFormPtr(event->data.frmLoad.formID);
      ListType *lt;
      int i;
      UInt16 dateIndex = 0, keywordIndex = 0, trackIndex = 0;

      /*
        Need to do this here, as JumpToRecord wants to update the visited link status
        of the document after the call to FrmGotoForm.
      */
      CloseDocument();

      /*
        Initialise the tracks popup.
      */
      lt = FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmSessionSearchTrackList));
      LstEraseList(lt);
      trackPtrs = MemPtrNew(numTracks * sizeof(Char *));
      for (i = 0; i < numTracks; i++)
      {
        trackPtrs[i] = (Char *)tracks[i].m_description;
        if (StrCompare(tracks[i].m_track, sessionSearchData.m_track) == 0)
        {
          trackIndex = i;
        }
      }

      LstSetListChoices(lt,trackPtrs,numTracks);
      LstSetTopItem(lt,trackIndex);
      LstSetSelection(lt,trackIndex);
      CtlSetLabel(FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmSessionSearchTrackPopup)),tracks[trackIndex].m_description);

      /*
        Initialise the keywords popup.
      */
      lt = FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmSessionSearchKeywordList));
      LstEraseList(lt);
      keywordPtrs = MemPtrNew(numKeywords * sizeof(Char *));
      for (i = 0; i < numKeywords; i++)
      {
        keywordPtrs[i] = (Char *)keywords[i].m_description;
        if (StrCompare(keywords[i].m_keyword, sessionSearchData.m_keyword) == 0)
        {
          keywordIndex = i;
        }
      }

      LstSetListChoices(lt,keywordPtrs,numKeywords);
      LstSetTopItem(lt,keywordIndex);
      LstSetSelection(lt,keywordIndex);
      CtlSetLabel(FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmSessionSearchKeywordPopup)),keywords[keywordIndex].m_description);

      /*
        Initialise the dates popup.
      */
      lt = FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmSessionSearchDateList));
      LstEraseList(lt);
      datePtrs = MemPtrNew(numDates * sizeof(Char *));
      for (i = 0; i < numDates; i++)
      {
        datePtrs[i] = (Char *)dates[i].m_textDate;
        if ((dates[i].m_year == sessionSearchData.m_year) &&
            (dates[i].m_month == sessionSearchData.m_month) &&
            (dates[i].m_day == sessionSearchData.m_day))
        {
          dateIndex = i;
        }
      }

      LstSetListChoices(lt,datePtrs,numDates);
      LstSetTopItem(lt,dateIndex);
      LstSetSelection(lt,dateIndex);
      CtlSetLabel(FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmSessionSearchDatePopup)),dates[dateIndex].m_textDate);


//FIXME      frmSessionSearchCMEEdit

      FrmSetActiveForm(ft);
      handled = true;
      break;
    }
    case frmOpenEvent:
    {
      FormType *ft = FrmGetActiveForm();

      FrmDrawForm(ft);
      handled = true;
      break;
    }
    case frmCloseEvent:
    {
      MemPtrFree(trackPtrs);
      MemPtrFree(keywordPtrs);
      MemPtrFree(datePtrs);
      break;
    }
    case penUpEvent:
    {
      RectangleType bounds;
      bounds.topLeft.x = 0;
      bounds.topLeft.y = 0;
      bounds.extent.x = 160;
      bounds.extent.y = 16;

      if (RctPtInRectangle( event->screenX, event->screenY, &bounds ))
      {
        prevCoordSys = PalmSetCoordinateSystem( NATIVE );
        TranslateHiResPenEvents( event );
        HandlePenUp( event->screenX, event->screenY );
        PalmSetCoordinateSystem( prevCoordSys );
      }
      break;
    }
    case penDownEvent:
    {
      RectangleType bounds;
      bounds.topLeft.x = 0;
      bounds.topLeft.y = 0;
      bounds.extent.x = 160;
      bounds.extent.y = 16;

      if (RctPtInRectangle( event->screenX, event->screenY, &bounds ))
      {
        prevCoordSys = PalmSetCoordinateSystem( NATIVE );
        TranslateHiResPenEvents( event );
        HandlePenDown( event->screenX, event->screenY );
        PalmSetCoordinateSystem( prevCoordSys );
      }
      break;
    }
    case ctlSelectEvent:
    {
      if ( event->data.ctlEnter.controlID == frmSessionSearchSearchButton)
      {
        FormType *ft = FrmGetActiveForm();
        Int16 track,keyword,date;
        Char *cme;
        SessionSearch *ss;

        track = LstGetSelection(FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmSessionSearchTrackList)));
        keyword = LstGetSelection(FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmSessionSearchKeywordList)));
        date = LstGetSelection(FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmSessionSearchDateList)));
        cme = FldGetTextPtr(FrmGetObjectPtr(ft, FrmGetObjectIndex(ft, frmSessionSearchCMEEdit)));
        if (cme == NULL)
        {
          cme = "";
        }

        SetHistorySessionSearch(dates[date].m_year,
                                dates[date].m_month,
                                dates[date].m_day,
                                keywords[keyword].m_keyword,
                                tracks[track].m_track, 
                                cme);

        ss = SessionSearch_new(dates[date].m_year,
                               dates[date].m_month,
                               dates[date].m_day,
                               keywords[keyword].m_keyword,
                               tracks[track].m_track, 
                               cme);

        
        if (ss->m_itemsCount == 0)
        {
          FrmAlert(infoNoSessionsFound);
        }
        else
        {
          Int16 docIndex;
          DocumentInfo *docInfo;
          SessionSearch_generatePlucker(ss);

          InitializeDocInfoList();
          FindDocData( SHOWGUIDE_TEMP_NAME, ALL_ELEMENTS, &docIndex );
          docInfo = DocInfo( docIndex );
          if (docInfo != NULL)
          {
            OpenDocument( docInfo );
            AddToHistorySessionList(dates[date].m_year,
                                    dates[date].m_month,
                                    dates[date].m_day,
                                    keywords[keyword].m_keyword,
                                    tracks[track].m_track, 
                                    cme);

            FrmGotoForm( GetMainFormId() );
          }
        }
        SessionSearch_delete(ss);

        handled = true;
      }
      break;
    }
    default:
    {
      break;
    }
  }

  return handled;
}
