#include "SessionSearchScreen.h"
#include "SessionSearch.h"
#include "reader-toolbar.h"
#include "resrc1.h"
#include "PDAVShowGuide.h"

/*
  FIXME: these should all be determined from databases.
*/
static DateMapper dates[]=
{
  {"Any", 0, 0},
  {"Thursday, May 26, 2005", 5, 26},
  {"Friday, May 27, 2005", 5, 27},
  {"Saturday, May 28, 2005", 5, 28},
  {"Sunday, May 29, 2005", 5, 29},
  {"Monday, May 30, 2005", 5, 30},
  {"Tuesday, May 31, 2005", 5, 31},
  {"Wednesday, June 1, 2005", 6, 1},
  {"Thursday, June 2, 2005", 6, 2}
};

static Track tracks[]=
{
  {"", "Any"},
  {"Knowledge","Knowledge Partner"},
  {"Specialty","Specialty"},
  {"HP2010","HP2010"},
  {"Primary","Primary Care"},
  {"Athletic","Athletic Trainer Credit"},
  {"CSAC","CSAC Top 10"},
  {"Adherance","Adherance"}
};

static Keyword keywords[]=
{
  {"", "Any"},
  {"ADJ","??? Unknown - ADJ ???"},
  {"ALRG","Allergy/immunology"},
  {"ALTE","Alternative/complementary med."},
  {"ANES","Anesthesia"},
  {"CARD","Cardiology"},
  {"CCME","Critical care medicine"},
  {"COMM","Communications"},
  {"DERM","Dermatology"},
  {"DIVR","Diversity"},
  {"EDUC","Education"},
  {"EMER","Emergency medicine"},
  {"ENDO","Endocrinology"},
  {"ENTH","Ears, nose, throat"},
  {"ETHI","Ethics"},
  {"GAST","Gastroenterology"},
  {"GENE","Genetics"},
  {"GERI","Geriatrics"},
  {"HEMO","Hematology"},
  {"HIVA","HIV/AIDS"},
  {"HPOL","Health policy"},
  {"INFD","Infectious diseases"},
  {"LABM","Lab medicine"},
  {"LEGI","Legislative issues"},
  {"MALP","Malpractice issues"},
  {"MISC","Miscellaneous"},
  {"NEPH","Nephrology"},
  {"NEUR","Neurology"},
  {"NUTR","Nutrition"},
  {"OBGY","Obstetrics/gynecology"},
  {"OCCU","Occupational/environmental med."},
  {"ONCO","Oncology"},
  {"OPHT","Ophthalmology"},
  {"ORTH","Orthopedics"},
  {"PALL","Palliative medicine"},
  {"PEDS","Pediatrics"},
  {"PHAR","Pharmacotherapeutics"},
  {"PREV","Preventive medicine"},
  {"PSYC","Psychiatry"},
  {"PULM","Pulmonology"},
  {"QUAS","Quality assurance"},
  {"RADI","Radiology"},
  {"REHB","Physical medicine/Rehab."},
  {"REIM","Reimbursement issues"},
  {"RHEU","Rheumatology"},
  {"RKMG","Risk management"},
  {"ROLE","Role of a PA"},
  {"SUBA","Substance abuse"},
  {"SURG","Surgery"},
  {"TECH","Technology"},
  {"TRAU","Trauma"},
  {"TRAV","Travel medicine"},
  {"UROL","Urology"},
  {"WS","Workshop"}
};


static void SessionSearchScreen_cstrToTstr(const char *cstr, LPTSTR tstr, UInt32 bufSize)
{
  UInt32 i;
  for (i = 0; i < bufSize-1; i++)
  {
    if (cstr[i] == '\0')
    {
      break;
    }
    tstr[i] = (unsigned char)cstr[i];
  }
  tstr[i] = '\0';
}

SessionSearchScreen *theSessionSearchScreen = NULL;

SessionSearchScreen *SessionSearchScreen_new(Library *library, RECT *windowRect)
{
  TCHAR buf[256+1];
  UInt32 i;
  SessionSearchScreen *self;

  int headingWidth = 60;
  int borderWidth = 10;

  self = malloc(sizeof(SessionSearchScreen));
  memset(self, 0, sizeof(SessionSearchScreen));

  self->m_library = library;

  self->m_keywords = keywords;
  self->m_keywordsCount = self->m_keywordsSize = sizeof(keywords)/sizeof(keywords[0]);

  self->m_tracks = tracks;
  self->m_tracksCount = self->m_tracksSize = sizeof(tracks)/sizeof(tracks[0]);

  self->m_dates = dates;
  self->m_datesCount = self->m_datesSize = sizeof(dates)/sizeof(dates[0]);




  LoadString(library->hInstance, IDS_SESSION_SEARCH_CLASS, buf, 256);

  self->m_hwnd = CreateWindow(buf, 
                              TEXT(""),
                              WS_VISIBLE,
                              windowRect->left,
                              windowRect->top,
                              windowRect->right - windowRect->left,
                              windowRect->bottom - windowRect->top,
                              NULL,
                              NULL,
                              library->hInstance,
                              NULL);


  LoadString(library->hInstance, IDS_SESSION_SEARCH_DESCRIPTION, buf, 256);

  self->m_statics[ST_DESCRIPTION] = CreateWindow(TEXT("Static"), 
                                                 buf, 
                                                 WS_VISIBLE|SS_LEFT,
                                                 borderWidth,
                                                 10,
                                                 windowRect->right - (windowRect->left + (2*borderWidth)),
                                                 20,
                                                 self->m_hwnd,
                                                 NULL,
                                                 library->hInstance,
                                                 NULL);

  LoadString(library->hInstance, IDS_SESSION_SEARCH_TRACK, buf, 256);

  self->m_statics[ST_TRACK] = CreateWindow(TEXT("Static"), 
                                           buf, 
                                           WS_VISIBLE|SS_LEFT,
                                           borderWidth,
                                           30,
                                           headingWidth,
                                           30,
                                           self->m_hwnd,
                                           NULL,
                                           library->hInstance,
                                           NULL);

  self->m_comboBoxes[CB_TRACK] = CreateWindow(TEXT("ComboBox"), 
                                           TEXT(""), 
                                           WS_VSCROLL|WS_VISIBLE|CBS_DROPDOWNLIST,
                                           borderWidth+headingWidth,
                                           30,
                                           windowRect->right - (windowRect->left+headingWidth+borderWidth+borderWidth),
                                           100,
                                           self->m_hwnd,
                                           NULL,
                                           library->hInstance,
                                           NULL);

  for (i = 0; i < self->m_tracksCount; i++)
  {
    SessionSearchScreen_cstrToTstr(self->m_tracks[i].m_description, buf, 256);
    SendMessage(self->m_comboBoxes[CB_TRACK],CB_ADDSTRING, 0, (LPARAM)buf);
  }
  SendMessage(self->m_comboBoxes[CB_TRACK],CB_SETCURSEL,0,0);


  LoadString(library->hInstance, IDS_SESSION_SEARCH_KEYWORD, buf, 256);

  self->m_statics[ST_KEYWORD] = CreateWindow(TEXT("Static"), 
                                           buf, 
                                           WS_VISIBLE|SS_LEFT,
                                           borderWidth,
                                           60,
                                           headingWidth,
                                           30,
                                           self->m_hwnd,
                                           NULL,
                                           library->hInstance,
                                           NULL);

  self->m_comboBoxes[CB_KEYWORD] = CreateWindow(TEXT("ComboBox"), 
                                           TEXT(""), 
                                           WS_VSCROLL|WS_VISIBLE|CBS_DROPDOWNLIST,
                                           borderWidth+headingWidth,
                                           60,
                                           windowRect->right - (windowRect->left+headingWidth+borderWidth+borderWidth),
                                           100,
                                           self->m_hwnd,
                                           NULL,
                                           library->hInstance,
                                           NULL);

  for (i = 0; i < self->m_keywordsCount; i++)
  {
    SessionSearchScreen_cstrToTstr(self->m_keywords[i].m_description, buf, 256);
    SendMessage(self->m_comboBoxes[CB_KEYWORD],CB_ADDSTRING, 0, (LPARAM)buf);
  }
  SendMessage(self->m_comboBoxes[CB_KEYWORD],CB_SETCURSEL,0,0);

  LoadString(library->hInstance, IDS_SESSION_SEARCH_DATE, buf, 256);

  self->m_statics[ST_DATE] = CreateWindow(TEXT("Static"), 
                                           buf, 
                                           WS_VISIBLE|SS_LEFT,
                                           borderWidth,
                                           90,
                                           headingWidth,
                                           30,
                                           self->m_hwnd,
                                           NULL,
                                           library->hInstance,
                                           NULL);

  self->m_comboBoxes[CB_DATE] = CreateWindow(TEXT("ComboBox"), 
                                           TEXT(""), 
                                           WS_VSCROLL|WS_VISIBLE|CBS_DROPDOWNLIST,
                                           borderWidth+headingWidth,
                                           90,
                                           windowRect->right - (windowRect->left+headingWidth+borderWidth+borderWidth),
                                           100,
                                           self->m_hwnd,
                                           NULL,
                                           library->hInstance,
                                           NULL);

  for (i = 0; i < self->m_datesCount; i++)
  {
    SessionSearchScreen_cstrToTstr(self->m_dates[i].m_textDate, buf, 256);
    SendMessage(self->m_comboBoxes[CB_DATE],CB_ADDSTRING, 0, (LPARAM)buf);
  }
  SendMessage(self->m_comboBoxes[CB_DATE],CB_SETCURSEL,0,0);

  LoadString(library->hInstance, IDS_SESSION_SEARCH_CME, buf, 256);

  self->m_statics[ST_CME] = CreateWindow(TEXT("Static"), 
                                           buf, 
                                           WS_VISIBLE|SS_LEFT,
                                           borderWidth,
                                           120,
                                           headingWidth,
                                           30,
                                           self->m_hwnd,
                                           NULL,
                                           library->hInstance,
                                           NULL);

  
  self->m_editBox = CreateWindow(TEXT("Edit"), 
                                 TEXT(""), 
                                 WS_VISIBLE|WS_BORDER|ES_LEFT,
                                 borderWidth+headingWidth,
                                 120,
                                 windowRect->right - (windowRect->left+headingWidth+borderWidth+borderWidth),
                                 22,
                                 self->m_hwnd,
                                 NULL,
                                 library->hInstance,
                                 NULL);



  LoadString(library->hInstance, IDS_SESSION_SEARCH_BUTTON, buf, 256);

  self->m_button = CreateWindow(TEXT("Button"),
                                buf, 
                                WS_VISIBLE,
                                80,
                                150,
                                80,
                                22,
                                self->m_hwnd,
                                NULL,
                                library->hInstance,
                                NULL);


  self->m_toolbar = CreateReaderToolbar(self->m_hwnd, library, false);

  ShowWindow(self->m_hwnd, SW_SHOW);
  UpdateWindow(self->m_hwnd);

  return self;
}

void SessionSearchScreen_delete(SessionSearchScreen *self)
{

  DestroyWindow(self->m_hwnd);

  if (self == theSessionSearchScreen)
  {
    theSessionSearchScreen = NULL;
  }

  free(self);
}


LRESULT CALLBACK SessionSearchScreen_WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  LRESULT lResult=0;
  Boolean handled = false;
  SessionSearchScreen *self = theSessionSearchScreen;

  if (message == WM_COMMAND)
  {
    WORD hi = HIWORD(wParam);
    WORD lo = LOWORD(wParam);

    if ((hi == BN_CLICKED) && (((HWND)lParam) == self->m_button))
    {
      TCHAR buf[256+1];
      Document *document;
      int track = SendMessage(self->m_comboBoxes[CB_TRACK], CB_GETCURSEL, 0, 0);
      int keyword = SendMessage(self->m_comboBoxes[CB_KEYWORD], CB_GETCURSEL, 0, 0);
      int date = SendMessage(self->m_comboBoxes[CB_DATE], CB_GETCURSEL, 0, 0);

      GetDlgItemText(self->m_hwnd, 1, buf, 256);

      theSessionSearch = SessionSearch_new(self->m_dates[date].m_month,
                                           self->m_dates[date].m_day,
                                           self->m_keywords[keyword].m_keyword,
                                           self->m_tracks[track].m_track, 
                                           "");

      SessionSearch_generatePlucker(theSessionSearch, 0);


      document = Library_findDocument( self->m_library, TEMPORARY_PLUCKER_DB_NAME );
      if( document != NULL )
      {
        OpenDocument( self->m_library->hWnd, document );
        HistoryList_setHistoryItemSessionList(HistoryList_add( self->m_library->history), TEMPORARY_PLUCKER_DB_NAME, 0);
        SessionSearchScreen_delete(self);
      }
      handled = true;
    }
    else if ((lo == ID_PREVIOUS_ITEM) || (lo == ID_NEXT_ITEM))
    {
      Library *library = self->m_library;
      Boolean went = false;
      if (lo == ID_PREVIOUS_ITEM)
      {
        went = (HistoryList_goBack(library->history) != NULL);
      }
      else
      {
        went = (HistoryList_goForward(library->history) != NULL);
      }

      if (went)
      {
        SessionSearchScreen_delete(self);
        GoCurrent(library, NULL);
      }
      handled = true;
    }
    else if (lo == ID_GO_HOME)
    {
      Document *document;
      Library *library = self->m_library;
      document = Library_findDocument( library, library->home_document );
      if( document != NULL )
      {
        SessionSearchScreen_delete(self);
        OpenDocument( library->hWnd, document );
      }
      handled = true;
    }
  }

  if (!handled)
  {
    lResult=DefWindowProc(hWnd, message, wParam, lParam);
  }

  return lResult;
}
