#define ALLOW_ACCESS_TO_INTERNALS_OF_BITMAPS
#include "SessionViewer.h"
#include "DataMgr.h"
#include "MemoryMgr.h"
#include <StringMgr.h>
#include "PluckerWriter.h"
#include "ShowSession.h"
#include "PDAVShowGuide.h"
#include "resourceids.h"
#include "prefsdata.h"
#include "hires.h"

static const char *static_headings[]=
{
    "Facility: ",
    "Room: ",
    "Speaker: ",
    "CME #: ",
    "Sponsorship: ",
    "Supported by: "
};

void SessionViewer_new(UInt16 id)
{
    MemHandle hrec;
    const UInt8 *prec;
    DmOpenRef db;
    UInt32 len;
    UInt16 paragraphLen = 0;
    LocalID indexRecordLocalID;
    LocalID homeHTMLRecordLocalID;
    LocalID urlHandlingRecordLocalID;
    LocalID urlDataRecordLocalID;
    LocalID imageDataRecordLocalID;
    const char *urlTransport = "event:";
    UInt16 urlLen;
    HiRes hiRes = HiResType();
    UInt16 screenDepth = Prefs()->screenDepth;
    BitmapPtr image;
    BitmapPtr tmpImage;
    Coord w;
    MemHandle imageHandle;
    UInt16 imageSize;
    UInt16 imageHeaderSize;
    UInt16 imageBitsSize;
    UInt8 bpp;
    UInt16 sz;

    const char *sessionData[6];
    Session session;
    int i;
    PluckerWriterReserved reserved[]=
    {
        {PLUCKER_RESERVED_HOME_HTML, 2},
        {PLUCKER_RESERVED_URL_HANDLING, 3},
    };

#define ARRAY_ELEMENTS(x) sizeof(x)/sizeof(x[0])

    if ((hiRes == unknownHiRes) ||
        (hiRes == noHiRes))
    {
      if (screenDepth >= 8)
      {
        bpp = 8;
      }
      else
      {
        bpp = 1;
      }
      sz = 16;
    }
    else
    {
      sz = 32;
      bpp = 8;
    }

    imageHandle = DmGetResource('Tbmp',bmpAppointment);
    tmpImage = (BitmapPtr)MemHandleLock(imageHandle);
    image = tmpImage;

    do
    {
      BmpGetDimensions(tmpImage, &w, NULL, NULL);
      if (w == sz)
      {
        if (BmpGetBitDepth(tmpImage) == bpp)
        {
          image = tmpImage;
          break;
        }
      }

      if (sz == 32)
      {
        tmpImage = BmpGetNextBitmapAnyDensity(tmpImage);
      }
      else
      {
        tmpImage = BmpGetNextBitmap(tmpImage);
      }
    } while (tmpImage != NULL);

    imageHeaderSize = BmpSize(image);
    if (image->flags.indirect)
    {
      imageBitsSize = BmpBitsSize(image);
    }
    else
    {
      imageBitsSize = 0;
    }
    imageSize = imageHeaderSize + imageBitsSize;


    db = DmOpenDatabaseByTypeCreator(SHOWGUIDE_SESSIONS_DB_TYPE,SHOWGUIDE_CREATOR,dmModeReadOnly);
    ErrFatalDisplayIf((db==NULL), "Failed to open Sessions database");

    hrec = DmQueryRecord(db,id);
    prec = MemHandleLock(hrec);

    /*
        Basic header and Record List
    */
    len = PLUCKER_WRITER_FILE_HEADER_LENGTH+2;
    len += PLUCKER_WRITER_RECORD_ENTRY_LENGTH * (3+(ARRAY_ELEMENTS(reserved)));

    indexRecordLocalID = len;
    len += PLUCKER_WRITER_INDEX_RECORD_LENGTH(ARRAY_ELEMENTS(reserved));

    homeHTMLRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += PLUCKER_WRITER_TEXT_DATA_RECORD_PARAGRAPH_HEADER_LENGTH*1;

    paragraphLen = 2*PLUCKER_CTL_LEN_SET_TEXT_COLOR;
    paragraphLen += 7*(PLUCKER_CTL_LEN_SET_FONT+PLUCKER_CTL_LEN_SET_FONT+PLUCKER_CTL_LEN_NEW_LINE);

    for (i = 0; i < 6; i++)
    {
        paragraphLen += StrLen(static_headings[i]);
    }
    
    session.m_eventNo = prec;

#define SESSION_FIELD(dest,src) session.dest = session.src + StrLen(session.src)+1; paragraphLen += (session.dest-session.src)-1;
#define SESSION_FIELD_DEST_HIDDEN(dest,src) session.dest = session.src + StrLen(session.src)+1;

    SESSION_FIELD_DEST_HIDDEN(m_date,m_eventNo)
    paragraphLen += PLUCKER_CTL_LEN_NEW_LINE;

    SESSION_FIELD(m_year,m_date)
    SESSION_FIELD_DEST_HIDDEN(m_month, m_year)
    SESSION_FIELD_DEST_HIDDEN(m_day, m_month)
    SESSION_FIELD_DEST_HIDDEN(m_startText,m_day)
    SESSION_FIELD(m_endText,m_startText)
    paragraphLen += 3; /* " - " */

    /*
        FIXME: this should be an image record at some point.
    */
    paragraphLen += PLUCKER_CTL_LEN_PAGE_LINK_BEGINS;
    paragraphLen += PLUCKER_CTL_LEN_EMBEDDED_IMAGE;
    paragraphLen += PLUCKER_CTL_LEN_LINK_ENDS;

    paragraphLen += PLUCKER_CTL_LEN_NEW_LINE;

    SESSION_FIELD(m_startTime,m_endText)
    SESSION_FIELD_DEST_HIDDEN(m_endTime,m_startTime)
    SESSION_FIELD_DEST_HIDDEN(m_cme,m_endTime)
    SESSION_FIELD(m_keyword,m_cme)
    SESSION_FIELD_DEST_HIDDEN(m_event,m_keyword)
    SESSION_FIELD(m_sponsorship,m_event)
    SESSION_FIELD(m_supportedBy,m_sponsorship)
    SESSION_FIELD(m_fullName,m_supportedBy)
    SESSION_FIELD(m_facility,m_fullName)
    SESSION_FIELD(m_location,m_facility)
    SESSION_FIELD(m_seqNum,m_location)
    SESSION_FIELD_DEST_HIDDEN(m_track,m_seqNum)


    sessionData[0] = session.m_facility;
    sessionData[1] = session.m_location;
    sessionData[2] = session.m_fullName;
    sessionData[3] = session.m_cme;
    sessionData[4] = session.m_sponsorship;
    sessionData[5] = session.m_supportedBy;


    len += paragraphLen;

    /*
        Data size all taken in to account. Now work out the size for the URLs
    */
    urlHandlingRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += PLUCKER_WRITER_URL_HANDLING_DATA_RECORD_PARAGRAPH_HEADER_LENGTH;

    urlDataRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += 5; // 4 zero terminators for unused records
    urlLen = StrLen(urlTransport)+StrLen(session.m_year)+StrLen(session.m_month)+StrLen(session.m_day)+StrLen(session.m_startTime)+StrLen(session.m_endTime)+StrLen(session.m_facility)+StrLen(session.m_location)+StrLen(session.m_event)+7+1; /* "year~month~day~start~end~facility~location~description" */
    len += urlLen;

    imageDataRecordLocalID = len;
    len += PLUCKER_WRITER_DATA_RECORD_HEADER_LENGTH;
    len += imageSize;

    {


        UInt8 *dbPtr = MemPtrNew(len);
        UInt8 *tmp = dbPtr;
        tmp = PluckerWriter_fileHeader(tmp, 1, NULL);
        tmp = PluckerWriter_uInt16(tmp, 5); /* num record entries */
        tmp = PluckerWriter_recordEntry(tmp, indexRecordLocalID, 0, 0);
        tmp = PluckerWriter_recordEntry(tmp, homeHTMLRecordLocalID, 0, 1);
        tmp = PluckerWriter_recordEntry(tmp, urlHandlingRecordLocalID, 0, 2);
        tmp = PluckerWriter_recordEntry(tmp, urlDataRecordLocalID, 0, 3);
        tmp = PluckerWriter_recordEntry(tmp, imageDataRecordLocalID, 0, 4);
        tmp = PluckerWriter_indexRecord(tmp, ARRAY_ELEMENTS(reserved), reserved);
        tmp = PluckerWriter_dataRecordHeader(tmp, reserved[0].id, 1, paragraphLen+(PLUCKER_WRITER_TEXT_DATA_RECORD_PARAGRAPH_HEADER_LENGTH*1), PLUCKER_DATATYPE_PHTML);
        tmp = PluckerWriter_uInt16(tmp, paragraphLen);
        tmp = PluckerWriter_uInt16(tmp, 0);

        tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_BOLD);
        tmp = PluckerWriter_setTextColor(tmp, 0xff, 0x80, 0x00);
        tmp = PluckerWriter_writeZTS(tmp, session.m_event);
        tmp = PluckerWriter_setTextColor(tmp, 0x00, 0x00, 0x00);
        tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_REGULAR);
        tmp = PluckerWriter_newLine(tmp);


        tmp = PluckerWriter_writeZTS(tmp, session.m_date);
        tmp = PluckerWriter_newLine(tmp);
        tmp = PluckerWriter_writeZTS(tmp, session.m_startText);
        tmp = PluckerWriter_writeZTS(tmp, " - ");
        tmp = PluckerWriter_writeZTS(tmp, session.m_endText);
        tmp = PluckerWriter_pageLinkBegins(tmp, 6);
        tmp = PluckerWriter_embeddedImage(tmp, 5);
        tmp = PluckerWriter_linkEnds(tmp);
        tmp = PluckerWriter_newLine(tmp);


        for (i = 0; i < 6; i++)
        {
            tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_BOLD);
            tmp = PluckerWriter_writeZTS(tmp, static_headings[i]);
            tmp = PluckerWriter_setFont(tmp, PLUCKER_FONT_REGULAR);
            tmp = PluckerWriter_writeZTS(tmp, sessionData[i]);
            tmp = PluckerWriter_newLine(tmp);
        }

        /*
            URL handling record.
        */
        tmp = PluckerWriter_dataRecordHeader(tmp, reserved[1].id, 0, PLUCKER_WRITER_URL_HANDLING_DATA_RECORD_PARAGRAPH_HEADER_LENGTH, PLUCKER_DATATYPE_LINK_INDEX);
        tmp = PluckerWriter_uInt16(tmp, 6);
        tmp = PluckerWriter_uInt16(tmp, 4);

        /*
            URL data record.
        */
        tmp = PluckerWriter_dataRecordHeader(tmp, 4, 0, urlLen+5, PLUCKER_DATATYPE_LINKS);
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 1)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 2)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 3)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 4)
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator (URL 5)
        tmp = PluckerWriter_writeZTS(tmp, urlTransport);        
        tmp = PluckerWriter_writeZTS(tmp, session.m_year);
        tmp = PluckerWriter_uInt8(tmp, '~');
        tmp = PluckerWriter_writeZTS(tmp, session.m_month);
        tmp = PluckerWriter_uInt8(tmp, '~');
        tmp = PluckerWriter_writeZTS(tmp, session.m_day);
        tmp = PluckerWriter_uInt8(tmp, '~');
        tmp = PluckerWriter_writeZTS(tmp, session.m_startTime);
        tmp = PluckerWriter_uInt8(tmp, '~');
        tmp = PluckerWriter_writeZTS(tmp, session.m_endTime);
        tmp = PluckerWriter_uInt8(tmp, '~');
        tmp = PluckerWriter_writeZTS(tmp, session.m_facility);
        tmp = PluckerWriter_uInt8(tmp, '~');
        tmp = PluckerWriter_writeZTS(tmp, session.m_location);
        tmp = PluckerWriter_uInt8(tmp, '~');
        tmp = PluckerWriter_writeZTS(tmp, session.m_event);
        tmp = PluckerWriter_uInt8(tmp, 0); // NUL terminator

        /*
          Image record.
        */
        tmp = PluckerWriter_dataRecordHeader(tmp, 5, 0, imageSize, PLUCKER_DATATYPE_TBMP);
        tmp = PluckerWriter_writeBlock(tmp, image, imageHeaderSize);
        if (imageBitsSize)
        {
          tmp = PluckerWriter_writeBlock(tmp, BmpGetBits(image), imageBitsSize);
        }

        MemHandleUnlock(imageHandle);
        DmReleaseResource(imageHandle);
        
        {
            int dbLen = tmp - dbPtr;
            dbLen = dbLen;
        }
        PluckerWriter_createDatabaseFromImage(dbPtr);
        MemPtrFree(dbPtr);
    }

    MemHandleUnlock(hrec);
    DmCloseDatabase(db);
}

