/*
 * $Id: history->c,v 1.34 2003/12/29 20:22:29 nordstrom Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include "debug.h"
#include "document.h"
#include "genericfile.h"
#include "util.h"

#include "history.h"
#include "PDAVShowGuide.h"


/***********************************************************************
 *
 *      Local functions
 *
 ***********************************************************************/
static void RollHistoryIndex( Int16* index ) HISTORY_SECTION;
#ifndef PDAV_SHOWGUIDE
static void RestoreData( void ) HISTORY_SECTION;
#endif

/***********************************************************************
 *
 *      Private variables
 *
 ***********************************************************************/
static History *history;



/* Return next history index */
static void RollHistoryIndex
    (
    Int16* index    /* pointer to history index ( upon return, it will
                       contain the next history index ) */
    )
{
    ( *index )++;
    if ( HISTORYSIZE <= *index )
        *index = 0;
}

#ifndef PDAV_SHOWGUIDE

/* Restore data for current record from history */
static void RestoreData( void )
{
    MetaRecord* meta;
    MemHandle   handle;
    UInt16      recordId;

    recordId    = history->records[ history->currentRecord ].recordId;
    handle      = GetMetaHandle( recordId, false );
    meta        = MemHandleLock( handle );

    DmWrite( meta, OFFSETOF( MetaRecord, verticalOffset ),
        &history->records[ history->currentRecord ],
        sizeof( HistoryData ) - sizeof( Int16 ) );

    MemHandleUnlock( handle );
}

#endif

/* Initialize the history structure */
void InitHistory( void )
{
  if (history != NULL)
  {
    MemPtrFree(history);
  }

  history = MemPtrNew(sizeof(History));

  MemSet( history, sizeof( History ), 0 );

  history->addHistory              = false;
  history->records[ 0 ].type = HISTORY_ITEM_TYPE_PLUCKER;
  history->records[ 0 ].item.plucker.record   = HOME_PAGE_ID;
  StrCopy(history->records[ 0 ].databaseName, SHOWGUIDE_HOME_NAME);
  MemMove(&history->homeRecord, &history->records[0],sizeof(HistoryData));
}



/* Read history from session record */
void ReadHistory
    (
    History* recordPtr,     /* pointer to session record */
    UInt16   availableSize  /* amount of data available */
    )
{
  if (history == NULL)
  {
    InitHistory();
  }
    MemSet( history, sizeof( History ), 0 );
    if ( sizeof( History ) < availableSize )
        availableSize = sizeof( History );
    if ( 0 < availableSize )
        MemMove( history, recordPtr, availableSize );
}



/* Save history in session record */
void SaveHistory
    (
    History* recordPtr /* pointer to session record */
    )
{
    DmWrite( recordPtr, 0, history, sizeof( History ) );
}


#ifdef PDAV_SHOWGUIDE
/* Add record to history */
void AddToHistoryPlucker
    (
    const UInt16 recordId,  /* record ID */
    const Char *databaseName
    )
{
    if ( ! history->addHistory ) {
        history->addHistory = true;
        return;
    }

    RollHistoryIndex( &history->currentRecord );
    history->lastRecord = history->currentRecord;

    if ( history->currentRecord == history->firstRecord )
        RollHistoryIndex( &history->firstRecord );



    history->records[ history->currentRecord ].type = HISTORY_ITEM_TYPE_PLUCKER;
    history->records[ history->currentRecord ].item.plucker.record = recordId;
    SetHistoryDatabaseName(databaseName);
}


void AddToHistoryExhibitor( const UInt16 id )
{
    RollHistoryIndex( &history->currentRecord );
    history->lastRecord = history->currentRecord;

    if ( history->currentRecord == history->firstRecord )
        RollHistoryIndex( &history->firstRecord );

    history->records[ history->currentRecord ].type = HISTORY_ITEM_TYPE_EXHIBITOR;
    history->records[ history->currentRecord ].item.exhibitor.id = id;
    SetHistoryDatabaseName(SHOWGUIDE_TEMP_NAME);
}

void AddToHistoryExhibitorList( const UInt16 startId, Char limitChar)
{
    RollHistoryIndex( &history->currentRecord );
    history->lastRecord = history->currentRecord;

    if ( history->currentRecord == history->firstRecord )
        RollHistoryIndex( &history->firstRecord );



    history->records[ history->currentRecord ].type = HISTORY_ITEM_TYPE_EXHIBITOR_LIST;
    history->records[ history->currentRecord ].item.exhibitorList.startId = startId;
    history->records[ history->currentRecord ].item.exhibitorList.limitChar = limitChar;
    SetHistoryDatabaseName(SHOWGUIDE_TEMP_NAME);
}

void AddToHistorySession( const UInt16 id)
{
    RollHistoryIndex( &history->currentRecord );
    history->lastRecord = history->currentRecord;

    if ( history->currentRecord == history->firstRecord )
        RollHistoryIndex( &history->firstRecord );



    history->records[ history->currentRecord ].type = HISTORY_ITEM_TYPE_SESSION;
    history->records[ history->currentRecord ].item.session.id = id;
    SetHistoryDatabaseName(SHOWGUIDE_TEMP_NAME);
}

void AddToHistorySessionList( const UInt16 year, const UInt16 month, const UInt16 day, const Char *keyword, const Char *track, const Char *cme)
{
    RollHistoryIndex( &history->currentRecord );
    history->lastRecord = history->currentRecord;

    if ( history->currentRecord == history->firstRecord )
        RollHistoryIndex( &history->firstRecord );

    history->records[ history->currentRecord ].type = HISTORY_ITEM_TYPE_SESSION_LIST;
    history->records[ history->currentRecord ].item.sessionList.year = year;
    history->records[ history->currentRecord ].item.sessionList.month = month;
    history->records[ history->currentRecord ].item.sessionList.day = day;
    StrCopy(history->records[ history->currentRecord ].item.sessionList.keyword, keyword);
    StrCopy(history->records[ history->currentRecord ].item.sessionList.track, track);
    StrCopy(history->records[ history->currentRecord ].item.sessionList.cme, cme);
    SetHistoryDatabaseName(SHOWGUIDE_TEMP_NAME);
}

void SetHistorySessionSearch( const UInt16 year, const UInt16 month, const UInt16 day, const Char *keyword, const Char *track, const Char *cme)
{
    history->records[ history->currentRecord ].type = HISTORY_ITEM_TYPE_SESSION_SEARCH;
    history->records[ history->currentRecord ].item.sessionSearch.year = year;
    history->records[ history->currentRecord ].item.sessionSearch.month = month;
    history->records[ history->currentRecord ].item.sessionSearch.day = day;
    StrCopy(history->records[ history->currentRecord ].item.sessionSearch.keyword, keyword);
    StrCopy(history->records[ history->currentRecord ].item.sessionSearch.track, track);
    StrCopy(history->records[ history->currentRecord ].item.sessionSearch.cme, cme);
    SetHistoryDatabaseName(SHOWGUIDE_TEMP_NAME);
}

void AddToHistorySessionSearch( const UInt16 year, const UInt16 month, const UInt16 day, const Char *keyword, const Char *track, const Char *cme)
{
    RollHistoryIndex( &history->currentRecord );
    history->lastRecord = history->currentRecord;

    if ( history->currentRecord == history->firstRecord )
        RollHistoryIndex( &history->firstRecord );

    SetHistorySessionSearch(year, month, day, keyword, track, cme);
}

void AddToHistoryMap(void)
{
    RollHistoryIndex( &history->currentRecord );
    history->lastRecord = history->currentRecord;

    if ( history->currentRecord == history->firstRecord )
        RollHistoryIndex( &history->firstRecord );

    history->records[ history->currentRecord ].type = HISTORY_ITEM_TYPE_MAP;
    SetHistoryDatabaseName("");
}

#else

/* Add record to history */
void AddToHistory
    (
    const UInt16 recordId  /* record ID */
    )
{
    if ( ! history->addHistory ) {
        history->addHistory = true;
        return;
    }

    RollHistoryIndex( &history->currentRecord );
    history->lastRecord = history->currentRecord;

    if ( history->currentRecord == history->firstRecord )
        RollHistoryIndex( &history->firstRecord );

    history->records[ history->currentRecord ].recordId = recordId;
}

#endif

/* Store data for first paragraph in current record from history */
void SetHistoryFirstParagraph
    (
    const Int16   offset, /* offset to paragraph in record */
    const YOffset y       /* paragraph's y coordinate within record */
    )
{
  if (history->records[ history->currentRecord ].type == HISTORY_ITEM_TYPE_PLUCKER)
  {
    history->records[ history->currentRecord ].item.plucker.y_offset = y;
  }
}



#ifdef STORE_LAST_VISIBLE
/* Store data for last paragraph in current record from history */
void SetHistoryLastParagraph
    (
    const Int16 offset,   /* offset to paragraph in record */
    const YOffset y       /* paragraph's y coordinate within record */
    )
{
    history->records[ history->currentRecord ].lastVisibleParagraph   = offset;
    history->records[ history->currentRecord ].lastParagraphY         = y;
}
#endif



/* Store vertical offset for current record from history */
void SetHistoryVerticalOffset
    (
    const YOffset offset  /* vertical offset in record */
    )
{
  if (history->records[ history->currentRecord ].type == HISTORY_ITEM_TYPE_PLUCKER)
  {
    history->records[ history->currentRecord ].item.plucker.y_offset = offset;
  }
}



/* Get the add history status */
const Boolean GetAddHistory( void )
{
    return history->addHistory;
}


#ifdef PDAV_SHOWGUIDE

/* Get the current record ID from the history */
const HistoryData *GetHistoryCurrent( void )
{
    return &history->records[ history->currentRecord ];
}

const HistoryData *GetHistoryHome( void )
{
    return &history->homeRecord;
}


/* Get the next record from the history */
const HistoryData *GetHistoryNext( void )
{
    if ( history->currentRecord == history->lastRecord )
        return NULL;

    RollHistoryIndex( &history->currentRecord );

//    RestoreData();

    return &history->records[ history->currentRecord ];
}



/* Get the previous record from the history */
const HistoryData *GetHistoryPrev( void )
{
    if ( history->currentRecord == history->firstRecord )
        return NULL;

    if ( --history->currentRecord < 0 )
        history->currentRecord = HISTORYSIZE - 1;

//    RestoreData();

    return &history->records[ history->currentRecord ];
}

#else
/* Get the current record ID from the history */
const UInt16 GetHistoryCurrent( void )
{
    return history->records[ history->currentRecord ].recordId;
}



/* Get the next record from the history */
const UInt16 GetHistoryNext( void )
{
    if ( history->currentRecord == history->lastRecord )
        return NO_RECORD;

    RollHistoryIndex( &history->currentRecord );

    RestoreData();

    return history->records[ history->currentRecord ].recordId;
}



/* Get the previous record from the history */
const UInt16 GetHistoryPrev( void )
{
    if ( history->currentRecord == history->firstRecord )
        return NO_RECORD;

    if ( --history->currentRecord < 0 )
        history->currentRecord = HISTORYSIZE - 1;

    RestoreData();

    return history->records[ history->currentRecord ].recordId;
}

#endif

/* get the history data */
History* GetHistoryPtr( void )
{
    return history;
}



/* reset verticalOffsets in the history */
void ResetHistoryVerticalOffsets( void )
{
    Int16 i;
    for ( i = 0 ; i < HISTORYSIZE ; i++ )
    {
      if (history->records[ i ].type == HISTORY_ITEM_TYPE_PLUCKER)
      {

        history->records[ i ].item.plucker.y_offset = NO_VERTICAL_OFFSET;
      }
    }
}



/* set character position of start */
void SetHistoryCharacterPosition( Int16 position )
{
  if (history->records[ history->currentRecord ].type == HISTORY_ITEM_TYPE_PLUCKER)
  {
    history->records[ history->currentRecord ].item.plucker.char_index = position;
  }
}



/* set character position of start */
Int16 GetHistoryCharacterPosition( void )
{
    return history->records[ history->currentRecord ].item.plucker.char_index;
}



/* set current record */
void SetHistoryRecordId( UInt16 recordId )
{
  if (history->records[ history->currentRecord ].type == HISTORY_ITEM_TYPE_PLUCKER)
  {
    history->records[ history->currentRecord ].item.plucker.record = recordId;
  }
}

void SetHistoryDatabaseName( const Char *databaseName)
{
    StrNCopy(history->records[ history->currentRecord ].databaseName, databaseName, dmDBNameLength);
}
