/*
 * $Id: history.h,v 1.30 2003/08/11 00:21:25 prussar Exp $
 *
 * Viewer - a part of Plucker, the free off-line HTML viewer for PalmOS
 * Copyright (c) 1998-2002, Mark Ian Lillywhite and Michael Nordstrom
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifndef PLUCKER_HISTORY_H
#define PLUCKER_HISTORY_H

#include "viewer.h"
#include "prefsdata.h"
#include "WindowsRect.h"

#define HISTORYSIZE 32



#ifdef PDAV_SHOWGUIDE

typedef enum
{
  HISTORY_ITEM_TYPE_PLUCKER,
  HISTORY_ITEM_TYPE_MAP,
  HISTORY_ITEM_TYPE_EXHIBITOR,
  HISTORY_ITEM_TYPE_EXHIBITOR_LIST,
  HISTORY_ITEM_TYPE_SESSION,
  HISTORY_ITEM_TYPE_SESSION_LIST,
  HISTORY_ITEM_TYPE_SESSION_SEARCH,

} HISTORY_ITEM_TYPE;


typedef struct
{
  UInt16 record;       /* the record index */
  Int16 char_index;   /* the index of the first character on the first full line on the screen */
  YOffset y_offset;     /* the yoffset from the top of the screen to the first full line on the screen */

}HISTORY_ITEM_PLUCKER;


typedef struct
{
  float zoomLevel;
  Int32 xCentre;
  Int32 yCentre;
  RECT highlightRect;

} HISTORY_ITEM_MAP;

typedef struct
{
  UInt16 id;

} HISTORY_ITEM_SESSION;

typedef struct
{
  UInt16 year;
  UInt16 month;
  UInt16 day;
  char keyword[4+1];
  char track[32+1];
  char cme[16+1];
} HISTORY_ITEM_SESSION_LIST;

typedef struct
{
  UInt16 year;
  UInt16 month;
  UInt16 day;
  char keyword[4+1];
  char track[32+1];
  char cme[16+1];
} HISTORY_ITEM_SESSION_SEARCH;

typedef struct
{
  UInt16 id;

} HISTORY_ITEM_EXHIBITOR;

typedef struct
{
  UInt16 startId;
  char limitChar;

} HISTORY_ITEM_EXHIBITOR_LIST;

#endif



/* The first four entries in this type must correspond to
   entries 2-5 in MetaRecord. */
typedef struct {
#ifndef PDAV_SHOWGUIDE
    YOffset verticalOffset;
    Int16   characterPosition;
    Int16   firstVisibleParagraph;
    YOffset firstParagraphY;
    UInt16  recordId;
#else
    Char    databaseName[dmDBNameLength];

    HISTORY_ITEM_TYPE type;

    union
    {
      HISTORY_ITEM_PLUCKER plucker;
      HISTORY_ITEM_MAP map;
      HISTORY_ITEM_SESSION session;
      HISTORY_ITEM_SESSION_LIST sessionList;
      HISTORY_ITEM_SESSION_SEARCH sessionSearch;
      HISTORY_ITEM_EXHIBITOR exhibitor;
      HISTORY_ITEM_EXHIBITOR_LIST exhibitorList;
    } item;
#endif


} HistoryData;


typedef struct {
    UInt16          uid;
    Int16           firstRecord;
    Int16           lastRecord;
    Int16           currentRecord;
    Boolean         addHistory;
    HistoryData     homeRecord;
    HistoryData     records[ HISTORYSIZE ];
    UInt16          depth;
    FontModeType    font;
    ToolbarType     toolbar;
    ScrollbarType   scrollbar;
    Int16           lineSpacing;
    Int16           paragraphSpacing;
    Char            userFontName[ dmDBNameLength ];
    UInt16          maxExtentX;
    RotateType      rotate;
    UInt16          extentX;
} History;


/* Initialize the history structure */
extern void InitHistory( void ) HISTORY_SECTION;

/* Read history from session record */
extern void ReadHistory( History* recordPtr, UInt16 availableSize ) 
        HISTORY_SECTION;

/* Save history in session record */
extern void SaveHistory( History* recordPtr ) HISTORY_SECTION;

/* Store data for first paragraph in current record from history */
extern void SetHistoryFirstParagraph( const Int16 offset,
        const YOffset y ) HISTORY_SECTION;

#ifdef STORE_LAST_VISIBLE
/* Store data for last paragraph in current record from history */
extern void SetHistoryLastParagraph( const Int16 offset,
        const YOffset y ) HISTORY_SECTION;
#endif

/* Store vertical offset for current record from history */
extern void SetHistoryVerticalOffset( const YOffset offset ) HISTORY_SECTION;

#ifdef PDAV_SHOWGUIDE
/* Add record to history */
extern void AddToHistoryPlucker( const UInt16 recordId, const Char *databaseName ) HISTORY_SECTION;
extern void AddToHistoryExhibitor( const UInt16 id ) HISTORY_SECTION;
extern void AddToHistoryExhibitorList( const UInt16 startId, Char limitChar) HISTORY_SECTION;
extern void AddToHistorySession( const UInt16 id) HISTORY_SECTION;
extern void AddToHistorySessionList( const UInt16 year, const UInt16 month, const UInt16 day, const Char *keyword, const Char *track, const Char *cme) HISTORY_SECTION;
extern void AddToHistorySessionSearch( const UInt16 year, const UInt16 month, const UInt16 day, const Char *keyword, const Char *track, const Char *cme) HISTORY_SECTION;
extern void SetHistorySessionSearch( const UInt16 year, const UInt16 month, const UInt16 day, const Char *keyword, const Char *track, const Char *cme) HISTORY_SECTION;
extern void AddToHistoryMap(void) HISTORY_SECTION;


/* Get the previous record from the history */
extern const HistoryData *GetHistoryPrev( void ) HISTORY_SECTION;

/* Get the next record from the history */
extern const HistoryData *GetHistoryNext( void ) HISTORY_SECTION;

/* Get the current record ID from the history */
extern const HistoryData *GetHistoryCurrent( void ) HISTORY_SECTION;

/* Get the home record from the history */
extern const HistoryData *GetHistoryHome( void ) HISTORY_SECTION;

#else
/* Add record to history */
extern void AddToHistory( const UInt16 recordId ) HISTORY_SECTION;

/* Get the previous record from the history */
extern const UInt16 GetHistoryPrev( void ) HISTORY_SECTION;

/* Get the next record from the history */
extern const UInt16 GetHistoryNext( void ) HISTORY_SECTION;

/* Get the current record ID from the history */
extern const UInt16 GetHistoryCurrent( void ) HISTORY_SECTION;

#endif

/* Get the add history status */
extern const Boolean GetAddHistory( void ) HISTORY_SECTION;

/* Get the history structure */
extern History* GetHistoryPtr( void ) HISTORY_SECTION;

/* reset verticalOffsets in the history */
extern void ResetHistoryVerticalOffsets( void ) HISTORY_SECTION;

/* set character position of start */
extern void SetHistoryCharacterPosition( Int16 position ) HISTORY_SECTION;

/* set character position of start */
extern Int16 GetHistoryCharacterPosition( void ) HISTORY_SECTION;

/* set the current recordid */
extern void SetHistoryRecordId( UInt16 recordId ) HISTORY_SECTION;

extern void SetHistoryDatabaseName( const Char *databaseName) HISTORY_SECTION;

#endif

